using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager
{
	public partial class CityTypesEditor : UserControl
	{
		private Scenario		m_oSelectedScenario;
		private bool			m_bIgnoreEvents = false;


		public CityTypesEditor()
		{
			InitializeComponent();

			//coh.Width = -2;
		}


		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Scenario SelectedScenario 
		{
			get { return m_oSelectedScenario; }
			set { m_oSelectedScenario = value; LoadCityTypes(); } 
		}


		private void LoadCityTypes()
		{
			lvwCityTypes.Items.Clear();

			if (m_oSelectedScenario != null)
			{
				SortedList<int, ScenarioCityType> oCityTypes = new SortedList<int, ScenarioCityType>();
				foreach (ScenarioCityType oCityType in m_oSelectedScenario.CityTypes.Values)
				{
					oCityTypes.Add(oCityType.Size, oCityType);
				}

				for (int i = 0; i < oCityTypes.Count; i++)
				{
					AddCityType(oCityTypes.Values[i], false);
				}
			}

			Program.SuspendPaint(this);
			
			LoadDetails();
			SetControls();
			
			if (lvwCityTypes.Items.Count > 0)
			{
				lvwCityTypes.FocusedItem = lvwCityTypes.Items[0];
				lvwCityTypes.Items[0].Selected = true;
			}

			Program.ResumePaint(this);
		}

		private ListViewItem AddCityType(ScenarioCityType cityType, bool insertAfter)
		{
			ListViewItem lvi = new ListViewItem(cityType.Name);
			lvi.SubItems.Add(cityType.Size.ToString());
			lvi.SubItems.Add(string.Format("{0}/{1}/{2}", cityType.SmallBuildings, cityType.MediumBuildings, cityType.LargeBuildings));
			lvi.Tag = cityType;

			if (insertAfter)
			{
				int iIndex = lvwCityTypes.SelectedIndices[0];
				lvwCityTypes.Items.Insert(iIndex + 1, lvi);
			}
			else 
			{
				lvwCityTypes.Items.Add(lvi);
			}

			return lvi;
		}



		private void lvwCityTypes_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwCityTypes.SelectedItems.Count > 0)
			{
				Program.SuspendPaint(this);

				LoadDetails();
				SetControls();

				Program.ResumePaint(this);
			}
		}

		private void LoadDetails()
		{
			cboInput.Items.Clear();
			cboOutput.Items.Clear();

			cboInput.Items.Add(ScenarioGood.NoGood);
			cboOutput.Items.Add(ScenarioGood.NoGood);

			if (SelectedScenario != null)
			{
				foreach (ScenarioGood oGood in SelectedScenario.Goods.Values)
				{
					cboInput.Items.Add(oGood);
					cboOutput.Items.Add(oGood);
				}
			}
		}

		private void SetControls()
		{
			m_bIgnoreEvents = true;

			lvwResources.Items.Clear();

			ScenarioCityType oCityType = SelectedCityType;
			if (oCityType != null)
			{
				txtName.Text = oCityType.Name;
				txtName.Enabled = true;
				txtName.ForeColor = Color.Black;

				txtSmall.Text = oCityType.SmallBuildings.ToString();
				txtSmall.Enabled = true;

				txtMedium.Text = oCityType.MediumBuildings.ToString();
				txtMedium.Enabled = true;

				txtLarge.Text = oCityType.LargeBuildings.ToString();
				txtLarge.Enabled = true;

				LoadResources();
				lvwResources.Enabled = true;

				mnuAddCityType.Enabled = true;
				mnuDeleteCityType.Enabled = true;
				mnuDuplicate.Enabled = true;
				mnuMoveDown.Enabled = CanMoveDown();
				mnuMoveUp.Enabled = CanMoveUp();
			}
			else
			{
				txtName.Text = "";
				txtName.Enabled = false;

				txtSmall.Text = "";
				txtSmall.Enabled = false;

				txtMedium.Text = "";
				txtMedium.Enabled = false;

				txtLarge.Text = "";
				txtLarge.Enabled = false;

				lvwResources.Enabled = false;

				mnuAddCityType.Enabled = true;
				mnuDeleteCityType.Enabled = false;
				mnuDuplicate.Enabled = false;
				mnuMoveDown.Enabled = false;
				mnuMoveUp.Enabled = false;
			}

			m_bIgnoreEvents = false;
		}

		private ScenarioCityType SelectedCityType
		{
			get
			{
				ListViewItem lvi = SelectedItem;
				if (lvi == null) return null;

				return (ScenarioCityType)lvi.Tag;
			}
		}

		private ListViewItem SelectedItem
		{
			get { return (lvwCityTypes.SelectedItems.Count > 0) ? lvwCityTypes.SelectedItems[0] : null; }
		}


		private void lvwCityTypes_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Insert:
					if (mnuAddCityType.Enabled)
					{
						AddCityType();
					}
					break;

				case Keys.Delete:
					if (mnuDeleteCityType.Enabled && (e.Modifiers == Keys.Shift))
					{
						DeleteCityType();
					}
					break;

				case Keys.D:
					if (mnuDuplicate.Enabled && (e.Modifiers == Keys.Control))
					{
						DuplicateCityType();
					}
					break;

				case Keys.Up:
					if ((e.Modifiers == Keys.Shift) || (e.Modifiers == Keys.Control))
					{
						if (CanMoveUp())
						{
							DoMoveUp();
							e.Handled = true;
						}
					}
					break;

				case Keys.Down:
					if ((e.Modifiers == Keys.Shift) || (e.Modifiers == Keys.Control))
					{
						if (CanMoveDown())
						{
							DoMoveDown();
							e.Handled = true;
						}
					}
					break;
			}
		}

		private void lvwCityTypes_KeyUp(object sender, KeyEventArgs e)
		{
			SetControls();
		}

		private void lvwCityTypes_MouseUp(object sender, MouseEventArgs e)
		{
			SetControls();
		}

		private void mnuAddCityType_Click(object sender, EventArgs e)
		{
			AddCityType();
		}

		private void mnuDeleteCityType_Click(object sender, EventArgs e)
		{
			DeleteCityType();
		}

		private void mnuDuplicate_Click(object sender, EventArgs e)
		{
			DuplicateCityType();
		}

		private void mnuMoveUp_Click(object sender, EventArgs e)
		{
			DoMoveUp();
		}

		private void mnuMoveDown_Click(object sender, EventArgs e)
		{
			DoMoveDown();
		}


		private void AddCityType()
		{
			ScenarioCityType oCityType = new ScenarioCityType(SelectedScenario.ScenarioSettings);
			AddCityType(oCityType, "New City Type", false);
		}

		private void DeleteCityType()
		{
			int iIndex = lvwCityTypes.SelectedIndices[0];

			SelectedScenario.CityTypes.Remove(SelectedCityType.Name);

			lvwCityTypes.Items.RemoveAt(iIndex);

			RenumberCityTypes();

			if (iIndex >= lvwCityTypes.Items.Count) iIndex--;
			if (iIndex >= 0)
			{
				lvwCityTypes.Items[iIndex].Selected = true;
				lvwCityTypes.FocusedItem = lvwCityTypes.Items[iIndex];
			}

			SetControls();

			lvwCityTypes.Focus();
		}

		private void DuplicateCityType()
		{
			ScenarioCityType oDuplicate = (ScenarioCityType)SelectedCityType.Clone();
			oDuplicate.RootSetting = SelectedScenario.ScenarioSettings;

			AddCityType(oDuplicate, oDuplicate.Name, true);
		}

		private void AddCityType(ScenarioCityType cityType, string name, bool insertAfter)
		{
			string sName = "";

			for (int i = 1; i < 9999; i++)
			{
				bool bFound = false;
				sName = name + ((i > 1) ? " " + i.ToString() : "");
				foreach (ScenarioCityType oExistingCityType in SelectedScenario.CityTypes.Values)
				{
					if (oExistingCityType.Name == sName)
					{
						bFound = true;
						break;
					}
				}

				if (!bFound) break;
			}

			cityType.Name = sName;
			cityType.Size = SelectedScenario.CityTypes.Count;

			SelectedScenario.CityTypes.Add(sName, cityType);
			ListViewItem lvi = AddCityType(cityType, insertAfter);

			lvwCityTypes.FocusedItem = lvi;
			lvi.Selected = true;

			lvwCityTypes.Focus();
		}

		private bool CanMoveUp()
		{
			return (SelectedItem.Index > 0);
		}

		private void DoMoveUp()
		{
			int iIndex = lvwCityTypes.SelectedIndices[0];
			ListViewItem lvi = lvwCityTypes.SelectedItems[0];

			lvwCityTypes.Items.Remove(lvi);
			lvwCityTypes.Items.Insert(iIndex - 1, lvi);

			RenumberCityTypes();

			lvwCityTypes.FocusedItem = lvi;
			lvi.Selected = true;

			SetControls();

			lvwCityTypes.Focus();
		}

		private bool CanMoveDown()
		{
			return (SelectedItem.Index < lvwCityTypes.Items.Count - 1);
		}

		private void DoMoveDown()
		{
			int iIndex = lvwCityTypes.SelectedIndices[0];
			ListViewItem lvi = lvwCityTypes.SelectedItems[0];

			lvwCityTypes.Items.Remove(lvi);
			lvwCityTypes.Items.Insert(iIndex + 1, lvi);

			RenumberCityTypes();

			lvwCityTypes.FocusedItem = lvi;
			lvi.Selected = true;

			SetControls();

			lvwCityTypes.Focus();
		}

		private void RenumberCityTypes()
		{
			for (int i = 0; i < lvwCityTypes.Items.Count; i++)
			{
				((ScenarioCityType)lvwCityTypes.Items[i].Tag).Size = i;
				lvwCityTypes.Items[i].SubItems[1].Text = i.ToString();
			}
		}

		private void UpdateBuildings()
		{
			ScenarioCityType oCityType = SelectedCityType;
			SelectedItem.SubItems[2].Text = string.Format("{0}/{1}/{2}", SelectedCityType.SmallBuildings, oCityType.MediumBuildings, oCityType.LargeBuildings);
		}

		private void LoadResources()
		{
			foreach (ScenarioCityType.Resource oResource in SelectedCityType.ProductionModifiers)
			{
				AddResource(oResource);
			}

			if (lvwResources.Items.Count > 0)
			{
				lvwResources.FocusedItem = lvwResources.Items[0];
				lvwResources.Items[0].Selected = true;
			}
			else
			{
				cboInput.SelectedItem = null;
				cboInput.Enabled = false;

				cboOutput.SelectedItem = null;
				cboOutput.Enabled = false;

				txtRatio.Text = "";
				txtRatio.Enabled = false;

				txtStorage.Text = "";
				txtStorage.Enabled = false;
			}

		}

		private ListViewItem AddResource(ScenarioCityType.Resource resource)
		{
			ListViewItem lvi = new ListViewItem(resource.Input.Name);
			lvi.SubItems.Add(resource.Output.Name);
			lvi.SubItems.Add(resource.InputOutputRatio.ToString());
			lvi.SubItems.Add(resource.MaxStorage.ToString());

			lvi.Tag = resource;

			lvwResources.Items.Add(lvi);

			return lvi;
		}


		private void txtName_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			string sNewName = txtName.Text.Trim();
			if (sNewName == "") return;

			if (SelectedCityType.Name == sNewName) return;

			bool bError = false;
			foreach (ScenarioCityType oCityType in SelectedScenario.CityTypes.Values)
			{
				if (oCityType == SelectedCityType) continue;

				if (oCityType.Name == sNewName)
				{
					bError = true;
					break;
				}
			}

			if (bError)
			{
				txtName.ForeColor = Color.Red;
			}
			else 
			{
				txtName.ForeColor = Color.Black;

				SelectedScenario.CityTypes.Remove(SelectedCityType.Name);
				SelectedCityType.Name = sNewName;
				SelectedScenario.CityTypes.Add(sNewName, SelectedCityType);
				SelectedItem.Text = sNewName;
			}
		}



		private void txtSmall_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedCityType == null) return;

			if (txtSmall.Text != "")
			{
				int nSmall;
				if (int.TryParse(txtSmall.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out nSmall))
				{
					SelectedCityType.SmallBuildings = nSmall;
					UpdateBuildings();
					return;
				}
			}
		}

		private void txtSmall_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if ((e.KeyChar < '0') || (e.KeyChar > '9'))
				{
					e.Handled = true;
				}
			}
		}

		private void txtMedium_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedCityType == null) return;

			if (txtMedium.Text != "")
			{
				int nMedium;
				if (int.TryParse(txtMedium.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out nMedium))
				{
					SelectedCityType.MediumBuildings = nMedium;
					UpdateBuildings();
					return;
				}
			}
		}

		private void txtMedium_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if ((e.KeyChar < '0') || (e.KeyChar > '9'))
				{
					e.Handled = true;
				}
			}
		}

		private void txtLarge_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedCityType == null) return;

			if (txtLarge.Text != "")
			{
				int nLarge;
				if (int.TryParse(txtLarge.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out nLarge))
				{
					SelectedCityType.LargeBuildings = nLarge;
					UpdateBuildings();
					return;
				}
			}
		}

		private void txtLarge_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if ((e.KeyChar < '0') || (e.KeyChar > '9'))
				{
					e.Handled = true;
				}
			}
		}

		private void lvwResources_SelectedIndexChanged(object sender, EventArgs e)
		{
			SetControls2();
		}

		private void SetControls2()
		{
			bool b = m_bIgnoreEvents;
			m_bIgnoreEvents = true;

			ScenarioCityType.Resource oResource = SelectedResource;
			if (oResource != null)
			{
				IndustryEditor.SelectResource(cboInput, oResource.Input);
				cboInput.Enabled = true;

				IndustryEditor.SelectResource(cboOutput, oResource.Output);
				cboOutput.Enabled = true;

				txtRatio.Text = oResource.InputOutputRatio.ToString();
				txtRatio.Enabled = true;

				txtStorage.Text = oResource.MaxStorage.ToString();
				txtStorage.Enabled = true;

				mnuAddResource.Enabled = true;
				mnuRemoveResource.Enabled = true;
			}
			else
			{
				cboInput.SelectedItem = null;
				cboInput.Enabled = false;

				cboOutput.SelectedItem = null;
				cboOutput.Enabled = false;

				txtRatio.Text = "";
				txtRatio.Enabled = false;

				txtStorage.Text = "";
				txtStorage.Enabled = false;

				mnuAddResource.Enabled = lvwResources.Enabled;
				mnuRemoveResource.Enabled = false;
			}

			m_bIgnoreEvents = b;
		}


		private void lvwResources_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Insert:
					if (mnuAddResource.Enabled)
					{
						AddResource();
					}
					break;

				case Keys.Delete:
					if (mnuRemoveResource.Enabled && (e.Modifiers == Keys.Shift))
					{
						RemoveResource();
					}
					break;
			}
		}

		private void lvwResources_KeyUp(object sender, KeyEventArgs e)
		{
			SetControls2();
		}

		private void lvwResources_MouseUp(object sender, MouseEventArgs e)
		{
			SetControls2();
		}

		private ScenarioCityType.Resource SelectedResource
		{
			get
			{
				ListViewItem lvi = SelectedItem2;
				if (lvi == null) return null;

				return (ScenarioCityType.Resource)lvi.Tag;
			}
		}

		private ListViewItem SelectedItem2
		{
			get { return (lvwResources.SelectedItems.Count > 0) ? lvwResources.SelectedItems[0] : null; }
		}


		private void mnuAddResource_Click(object sender, EventArgs e)
		{
			AddResource();
		}

		private void mnuRemoveResource_Click(object sender, EventArgs e)
		{
			RemoveResource();
		}

		private void AddResource()
		{
			ScenarioCityType.Resource oResource = new ScenarioCityType.Resource(m_oSelectedScenario.ScenarioSettings);
			SelectedCityType.ProductionModifiers.Add(oResource);

			oResource.Input = ScenarioGood.AnyGood;
			oResource.Output = ScenarioGood.AnyGood;
			oResource.InputOutputRatio = 1;
			oResource.MaxStorage = 4;

			ListViewItem lvi = AddResource(oResource);

			lvwResources.FocusedItem = lvi;
			lvi.Selected = true;

			lvwResources.Focus();
		}

		private void RemoveResource()
		{
			int iIndex = lvwResources.SelectedIndices[0];

			SelectedCityType.ProductionModifiers.RemoveAt(iIndex);
			lvwResources.Items.RemoveAt(iIndex);

			if (iIndex >= lvwResources.Items.Count) iIndex--;
			if (iIndex >= 0)
			{
				lvwResources.Items[iIndex].Selected = true;
				lvwResources.FocusedItem = lvwResources.Items[iIndex];
			}

			SetControls2();

			lvwResources.Focus();
		}

		private void cboInput_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedResource == null) return;

			SelectedResource.Input = (Good)cboInput.SelectedItem;
			SelectedItem2.Text = SelectedResource.Input.Name;
		}

		private void cboOutput_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedResource == null) return;

			SelectedResource.Output = (Good)cboOutput.SelectedItem;
			SelectedItem2.SubItems[1].Text = SelectedResource.Output.Name;
		}

		private void txtRatio_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedResource == null) return;

			if (txtRatio.Text != "")
			{
				float fRatio;
				if (float.TryParse(txtRatio.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out fRatio))
				{
					SelectedResource.InputOutputRatio = fRatio;
					SelectedItem2.SubItems[2].Text = fRatio.ToString();
				}
			}
		}

		private void txtStorage_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			if (SelectedResource == null) return;

			if (txtStorage.Text != "")
			{
				int nStorage;
				if (int.TryParse(txtStorage.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out nStorage))
				{
					SelectedResource.MaxStorage = nStorage;
					SelectedItem2.SubItems[3].Text = nStorage.ToString();
				}
			}
		}

		private void txtRatio_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if (((e.KeyChar < '0') || (e.KeyChar > '9')) && (e.KeyChar != '.') && (e.KeyChar != ','))
				{
					e.Handled = true;
				}
			}
		}

		private void txtStorage_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if ((e.KeyChar < '0') || (e.KeyChar > '9'))
				{
					e.Handled = true;
				}
			}
		}

	}
}
