using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager.Editors
{
	public partial class BuildStuffConditionEditor : ConditionEditorBase
	{
		private BuildStuffCondition m_oCondition = null;


		public BuildStuffConditionEditor()
		{
			InitializeComponent();
		}


		protected override void OnSetNode(object node)
		{
			base.OnSetNode(node);

			m_oCondition = (BuildStuffCondition)node;

			lvwList.Items.Clear();
			for (int i = 0; i < m_oCondition.BuildThings.Count; i++)
			{
				BuildStuffCondition.BuildThing oThing = m_oCondition.BuildThings[i];
				AddThing(oThing);
			}

			LoadCities();
			SetControls();

			if (lvwList.Items.Count > 0)
			{
				lvwList.FocusedItem = lvwList.Items[0];
				lvwList.Items[0].Selected = true;
			}
		}

		private ListViewItem AddThing(BuildStuffCondition.BuildThing thing)
		{
			ListViewItem lvi = new ListViewItem(thing.City.Name);
			lvi.SubItems.Add(thing.TerminalType);
			lvi.Tag = thing;

			lvwList.Items.Add(lvi);

			return lvi;
		}

		private void lvwList_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwList.SelectedItems.Count > 0)
			{
				LoadCities();
				SetControls();
			}
		}

		private void LoadCities()
		{
			Program.SuspendPaint(this);

			ListViewItem lviSelected = SelectedItem;

			cboCity.Items.Clear();
			foreach (ScenarioCity oCity in SelectedScenario.Cities.Values)
			{
				bool bUnused = true;
				foreach (ListViewItem lvi in lvwList.Items)
				{
					if ((lvi != lviSelected) && (lvi.Text == oCity.Name))
					{
						bUnused = false;
						break;
					}
				}

				if (bUnused)
				{
					cboCity.Items.Add(oCity);
				}
			}

			Program.ResumePaint(this);
		}

		private void SetControls()
		{
			BuildStuffCondition.BuildThing oThing = SelectedBuildThing;
			if (oThing != null)
			{
				cboCity.SelectedItem = oThing.City;
				cboCity.Enabled = true;

				cboType.SelectedItem = oThing.TerminalType;
				cboType.Enabled = true;

				mnuAddNew.Enabled = (cboCity.Items.Count > 0);
				mnuDelete.Enabled = true;
			}
			else 
			{
				cboCity.SelectedIndex = -1;
				cboCity.Enabled = false;

				cboType.SelectedIndex = -1;
				cboType.Enabled = false;

				mnuAddNew.Enabled = true;
				mnuDelete.Enabled = false;
			}
		}

		private void mnuAddNew_Click(object sender, EventArgs e)
		{
			AddNew();
		}

		private void AddNew()
		{
			BuildStuffCondition.BuildThing oThing = new BuildStuffCondition.BuildThing((ScenarioSettings)m_oCondition.RootSetting);

			foreach (ScenarioCity oCity in cboCity.Items)
			{
				if (oCity == cboCity.SelectedItem) continue;	//skip if it is current selection

				oThing.City = oCity;
				oThing.TerminalType = (string)cboType.Items[0];

				m_oCondition.BuildThings.Add(oThing);
				ListViewItem lviNew = AddThing(oThing);

				lvwList.SelectedItems.Clear();
				lviNew.Selected = true;
				lvwList.FocusedItem = lviNew;

				break;
			}

			LoadCities();
			SetControls();
			SetNodeText();

			lvwList.Focus();
		}

		private void mnuDelete_Click(object sender, EventArgs e)
		{
			Delete();
		}

		private void Delete()
		{
			int iIndex = lvwList.SelectedIndices[0];

			m_oCondition.BuildThings.RemoveAt(iIndex);
			lvwList.Items.RemoveAt(iIndex);

			if (iIndex >= lvwList.Items.Count) iIndex--;
			if (iIndex >= 0)
			{
				lvwList.Items[iIndex].Selected = true;
				lvwList.FocusedItem = lvwList.Items[iIndex];
			}

			SetControls();
			SetNodeText();

			lvwList.Focus();
		}

		private void cboCity_SelectedIndexChanged(object sender, EventArgs e)
		{
			BuildStuffCondition.BuildThing oThing = SelectedBuildThing;
			if (oThing != null)
			{
				if (oThing.City != (ScenarioCity)cboCity.SelectedItem)
				{
					oThing.City = (ScenarioCity)cboCity.SelectedItem;
					SelectedItem.Text = oThing.City.Name;
				}
			}

			SetControls();
			SetNodeText();
		}

		private void cboType_SelectedIndexChanged(object sender, EventArgs e)
		{
			BuildStuffCondition.BuildThing oThing = SelectedBuildThing;
			if (oThing != null)
			{
				if (oThing.TerminalType != (string)cboType.SelectedItem)
				{
					oThing.TerminalType = (string)cboType.SelectedItem;
					SelectedItem.SubItems[1].Text = oThing.TerminalType;
				}
			}

			SetControls();
			SetNodeText();
		}

		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);

			if (lvwList != null)
			{
				lvwList.Columns[1].Width = -2;
			}
		}


		private BuildStuffCondition.BuildThing SelectedBuildThing
		{
			get
			{
				ListViewItem lvi = SelectedItem;
				if (lvi == null) return null;
				
				return (BuildStuffCondition.BuildThing)lvi.Tag;
			}
		}

		private ListViewItem SelectedItem
		{
			get { return (lvwList.SelectedItems.Count > 0) ? lvwList.SelectedItems[0] : null; }
		}


		private void lvwList_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Insert:
					if (mnuAddNew.Enabled)
					{
						AddNew();
					}
					break;

				case Keys.Delete:
					if (mnuDelete.Enabled && (e.Modifiers == Keys.Shift))
					{
						Delete();
					}
					break;
			}
		}

		private void lvwList_KeyUp(object sender, KeyEventArgs e)
		{
			LoadCities();
			SetControls();
		}

		private void lvwList_MouseUp(object sender, MouseEventArgs e)
		{
			LoadCities();
			SetControls();
		}
	}
}
