using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager.Editors
{
	public partial class ConnectConditionEditor : ConditionEditorBase
	{
		private ConnectCondition m_oCondition = null;

		public ConnectConditionEditor()
		{
			InitializeComponent();
		}

		protected override void OnSetNode(object node)
		{
			base.OnSetNode(node);

			m_oCondition = (ConnectCondition)node;

			lstAvailable.Items.Clear();
			foreach (ScenarioCity oCity in SelectedScenario.Cities.Values)
			{
				if (m_oCondition.Cities.Contains(oCity)) continue;
				lstAvailable.Items.Add(oCity);
			}
			if (lstAvailable.Items.Count > 0) lstAvailable.SelectedIndices.Add(0);

			lstConnect.Items.Clear();
			foreach (ScenarioCity oCity in m_oCondition.Cities)
			{
				lstConnect.Items.Add(oCity);
			}
			if (lstConnect.Items.Count > 0) lstConnect.SelectedIndices.Add(0);

			SetControls();
		}

		private void SetControls()
		{
			btnAdd.Enabled = (lstAvailable.SelectedItems.Count > 0);
			btnRemove.Enabled = (lstConnect.SelectedItems.Count > 0);
		}


		private void lstConnect_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			RemoveCities();
		}

		private void lstAvailable_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			AddCities();
		}

		private void btnAdd_Click(object sender, EventArgs e)
		{
			AddCities();
		}

		private void btnRemove_Click(object sender, EventArgs e)
		{
			RemoveCities();
		}

		private void lstConnect_SelectedIndexChanged(object sender, EventArgs e)
		{
			SetControls();
		}

		private void lstAvailable_SelectedIndexChanged(object sender, EventArgs e)
		{
			SetControls();
		}


		private void AddCities()
		{
			MoveItems(lstAvailable, lstConnect, true);
		}

		private void RemoveCities()
		{
			MoveItems(lstConnect, lstAvailable, false);
		}


		private void MoveItems(ListBox source, ListBox target, bool add)
		{
			int iIndex = 0;

			target.SelectedItems.Clear();

			while (source.SelectedItems.Count > 0)
			{
				iIndex = source.SelectedIndices[0];

				ScenarioCity oCity = (ScenarioCity)source.SelectedItems[0];
				source.Items.Remove(oCity);

				target.Items.Add(oCity);
				target.SelectedItems.Add(oCity);

				if (add)
				{
					m_oCondition.Cities.Add(oCity);
				}
				else 
				{
					m_oCondition.Cities.Remove(oCity);
				}
			}

			if (iIndex >= source.Items.Count) iIndex--;
			if (iIndex >= 0) source.SelectedIndex = iIndex;

			base.SetNodeText();
		}

		private void lstConnect_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{ 
				case Keys.Enter:
				case Keys.Delete:
					RemoveCities();
					break;
			}
		}

		private void lstAvailable_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{ 
				case Keys.Enter:
				case Keys.Insert:
					AddCities();
					break;
			}
		}

		private void lstConnect_DrawItem(object sender, DrawItemEventArgs e)
		{
			DrawItem(lstConnect, e);
		}

		private void lstAvailable_DrawItem(object sender, DrawItemEventArgs e)
		{
			DrawItem(lstAvailable, e);
		}

		private void DrawItem(ListBox listBox, DrawItemEventArgs e)
		{
			Rectangle rc = e.Bounds;
			Color crBack = lstConnect.BackColor;
			Color crText = lstConnect.ForeColor;

			if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
			{
				if ((e.State & DrawItemState.Focus) == DrawItemState.Focus)
				{
					crBack = SystemColors.Highlight;
					crText = SystemColors.HighlightText;
				}
				else
				{
					crBack = SystemColors.Control;
				}
			}

			Brush br = new SolidBrush(crBack);
			e.Graphics.FillRectangle(br, rc);
			br.Dispose();

			TextRenderer.DrawText(e.Graphics, listBox.Items[e.Index].ToString(), listBox.Font, rc, crText, TextFormatFlags.Left | TextFormatFlags.Top);

			if ((e.State & DrawItemState.Focus) == DrawItemState.Focus)
			{
				e.DrawFocusRectangle();
			}
		}
	}
}
