using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;

namespace Railroads.ScenarioManager.Editors
{
	public partial class EditorBase : UserControl
	{
		private static EraEditor					ms_oEraEditor = null;
		private static BuildStuffConditionEditor	ms_oBuildStuffConditionEditor = null;
		private static ConnectConditionEditor		ms_oConnectConditionEditor = null;
		private static GrowCityConditionEditor		ms_oGrowCityConditionEditor = null;
		private static NetWorthConditionEditor		ms_oNetWorthConditionEditor = null;
		private static OwnStockConditionEditor		ms_oOwnStockConditionEditor = null;
		private static OwnStuffConditionEditor		ms_oOwnStuffConditionEditor = null;
		private static ShipGoodsConditionEditor		ms_oShipGoodsConditionEditor = null;
		private static SpeedRecordConditionEditor	ms_oSpeedRecordConditionEditor = null;


		protected bool			m_bIgnoreEvents = false;
		protected TreeNode		m_tn = null;
		private MainForm		m_frm = null;

		public EditorBase()
		{
			InitializeComponent();
		}


		public void SetNode(TreeNode treeNode, MainForm form)
		{
			m_bIgnoreEvents = true;

			m_frm = form;
			m_tn = treeNode;
			OnSetNode(treeNode.Tag);

			m_bIgnoreEvents = false;
		}

		protected Scenario SelectedScenario
		{
			get { return m_frm.SelectedScenario; }
		}


		protected virtual void OnSetNode(object node)
		{ 
		}

		protected void SetNodeText()
		{
			SetNodeText(m_tn.Tag.ToString());
		}

		protected void SetNodeText(string text)
		{
			if (m_tn.Text != text)
			{
				Program.SuspendPaint(m_tn.TreeView);

				m_tn.TreeView.BeginUpdate();
				m_tn.Text = text;
				m_tn.TreeView.EndUpdate();

				Program.ResumePaint(m_tn.TreeView);
			}
		}


		public static EditorBase GetEditor(TreeNode treeNode)
		{
			if (treeNode == null) return null;

			object o = treeNode.Tag;

			if (o is Era)
			{
				if (ms_oEraEditor == null) ms_oEraEditor = new EraEditor();
				return ms_oEraEditor;
			}

			if (o is BuildStuffCondition)
			{
				if (ms_oBuildStuffConditionEditor == null) ms_oBuildStuffConditionEditor = new BuildStuffConditionEditor();
				return ms_oBuildStuffConditionEditor;
			}

			if (o is ConnectCondition)
			{
				if (ms_oConnectConditionEditor == null) ms_oConnectConditionEditor = new ConnectConditionEditor();
				return ms_oConnectConditionEditor;
			}

			if (o is GrowCityCondition)
			{
				if (ms_oGrowCityConditionEditor == null) ms_oGrowCityConditionEditor = new GrowCityConditionEditor();
				return ms_oGrowCityConditionEditor;
			}

			if (o is NetWorthCondition)
			{
				if (ms_oNetWorthConditionEditor == null) ms_oNetWorthConditionEditor = new NetWorthConditionEditor();
				return ms_oNetWorthConditionEditor;
			}

			if (o is OwnStockCondition)
			{
				if (ms_oOwnStockConditionEditor == null) ms_oOwnStockConditionEditor = new OwnStockConditionEditor();
				return ms_oOwnStockConditionEditor;
			}

			if (o is OwnStuffCondition)
			{
				if (ms_oOwnStuffConditionEditor == null) ms_oOwnStuffConditionEditor = new OwnStuffConditionEditor();
				return ms_oOwnStuffConditionEditor;
			}

			if (o is ShipGoodsCondition)
			{
				if (ms_oShipGoodsConditionEditor == null) ms_oShipGoodsConditionEditor = new ShipGoodsConditionEditor();
				return ms_oShipGoodsConditionEditor;
			}

			if (o is SpeedRecordCondition)
			{
				if (ms_oSpeedRecordConditionEditor == null) ms_oSpeedRecordConditionEditor = new SpeedRecordConditionEditor();
				return ms_oSpeedRecordConditionEditor;
			}

			return null;
		}
	}



	public class GroupBox2 : GroupBox
	{
		public GroupBox2()
		{
			this.DoubleBuffered = true;
		}
	}



	public class ListView2 : ListView
	{
		public ListView2()
		{
			this.DoubleBuffered = true;
		}
	}


	public class TreeView2 : TreeView
	{ 
		public TreeView2()
		{
		}

		protected override void WndProc(ref Message msg)
		{
			// turn the erase background message into a null message
			if ((int)0x0014 == msg.Msg) //if message is is erase background
			{
				msg.Msg = (int)0x0000; //reset message to null
			}

			base.WndProc(ref msg);
		}
	}
}
