using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager.Editors
{
	public partial class EraEditor : EditorBase
	{
		private Era m_oEra = null;


		public EraEditor()
		{
			InitializeComponent();

			for (int i = Constants.MIN_ERA_YEAR; i <= Constants.MAX_ERA_YEAR; i++)
			{
				cboStartYear.Items.Add(i);
				cboEndYear.Items.Add(i);
			}

			for (int i = 1; i <= 12; i++)
			{
				cboStartMonth.Items.Add(i);
				cboEndMonth.Items.Add(i);
			}
		}

		protected override void OnSetNode(object node)
		{
			base.OnSetNode(node);

			m_oEra = (Era)node;

			if (m_oEra != null)
			{
				txtName.Text = m_oEra.EraName;
				txtDescription.Text = m_oEra.Description;
				txtVictoryText.Text = m_oEra.VictoryText;
				txtFailureText.Text = m_oEra.FailureText;

				cboStartYear.SelectedItem = m_oEra.StartYear;
				cboStartMonth.SelectedItem = m_oEra.StartMonth;
				cboEndYear.SelectedItem = m_oEra.EndYear;
				cboEndMonth.SelectedItem = m_oEra.EndMonth;
			}
			else 
			{
				txtName.Text = "";
				txtDescription.Text = "";
				txtVictoryText.Text = "";
				txtFailureText.Text = "";
			}
		}


		private void cboStartYear_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			m_oEra.StartYear = (int)cboStartYear.SelectedItem;
		}

		private void cboStartMonth_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			m_oEra.StartMonth = (int)cboStartMonth.SelectedItem;
		}

		private void cboEndYear_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			m_oEra.EndYear = (int)cboEndYear.SelectedItem;
		}

		private void cboEndMonth_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			m_oEra.EndMonth = (int)cboEndMonth.SelectedItem;
		}

		private void txtVictoryText_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			m_oEra.VictoryText = txtVictoryText.Text;
		}

		private void txtFailureText_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			m_oEra.FailureText = txtFailureText.Text;
		}

		private void txtDescription_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			m_oEra.Description = txtDescription.Text;
		}

		private void txtName_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;
			m_oEra.EraName = txtName.Text;

			SetNodeText(m_oEra.EraName);
		}
	}
}
