using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager.Editors
{
	public partial class GrowCityConditionEditor : ConditionEditorBase
	{
		private GrowCityCondition m_oCondition = null;


		public GrowCityConditionEditor()
		{
			InitializeComponent();
		}


		protected override void OnSetNode(object node)
		{
			base.OnSetNode(node);

			m_oCondition = (GrowCityCondition)node;

			lvwList.Items.Clear();
			for (int i = 0; i < m_oCondition.Cities.Count; i++)
			{
				GrowCityCondition.GrowCity oGrowCity = m_oCondition.Cities[i];
				AddGrowCity(oGrowCity);
			}

			LoadDetails();
			SetControls();

			if (lvwList.Items.Count > 0)
			{
				lvwList.FocusedItem = lvwList.Items[0];
				lvwList.Items[0].Selected = true;
			}
		}

		private ListViewItem AddGrowCity(GrowCityCondition.GrowCity growCity)
		{
			ListViewItem lvi = new ListViewItem(growCity.City.Name);
			lvi.SubItems.Add((growCity.GrowQuantity > 0) ? growCity.GrowQuantity.ToString() : "");
			lvi.SubItems.Add(growCity.InitialSize.Name);
			lvi.SubItems.Add(growCity.TargetSize.Name);
			lvi.Tag = growCity;

			lvwList.Items.Add(lvi);

			return lvi;
		}



		private void SetControls()
		{
			GrowCityCondition.GrowCity oGrowCity = SelectedGrowCity;
			if (oGrowCity != null)
			{
				cboCity.SelectedItem = oGrowCity.City;
				cboCity.Enabled = true;

				txtCount.Text = (oGrowCity.GrowQuantity > 0) ? oGrowCity.GrowQuantity.ToString() : "";
				txtCount.Enabled = (oGrowCity.City == ScenarioCity.AnyCity);

				cboInitialSize.SelectedItem = oGrowCity.InitialSize;
				cboInitialSize.Enabled = true;

				cboTargetSize.SelectedItem = oGrowCity.TargetSize;
				cboTargetSize.Enabled = true;

				mnuAddNew.Enabled = true;
				mnuDelete.Enabled = true;
			}
			else
			{
				cboCity.SelectedItem = null;
				cboCity.Enabled = false;

				txtCount.Text = "";
				txtCount.Enabled = false;

				cboInitialSize.SelectedItem = null;
				cboInitialSize.Enabled = false;

				cboTargetSize.SelectedItem = null;
				cboTargetSize.Enabled = false;

				mnuAddNew.Enabled = true;
				mnuDelete.Enabled = false;
			}
		}

		private GrowCityCondition.GrowCity SelectedGrowCity
		{
			get
			{
				ListViewItem lvi = SelectedItem;
				if (lvi == null) return null;

				return (GrowCityCondition.GrowCity)lvi.Tag;
			}
		}

		private ListViewItem SelectedItem
		{
			get { return (lvwList.SelectedItems.Count > 0) ? lvwList.SelectedItems[0] : null; }
		}


		private void LoadDetails()
		{
			Program.SuspendPaint(this);

			cboCity.Items.Clear();
			cboCity.Items.Add(ScenarioCity.AnyCity);
			foreach (ScenarioCity oCity in SelectedScenario.Cities.Values)
			{
				cboCity.Items.Add(oCity);
			}

			FormsHelper.PopulateCityTypesCombo(cboInitialSize, SelectedScenario, true);
			FormsHelper.PopulateCityTypesCombo(cboTargetSize, SelectedScenario, false);

			Program.ResumePaint(this);

		}


		private void lvwList_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwList.SelectedItems.Count > 0)
			{
				LoadDetails();
				SetControls();
			}
		}

		private void lvwList_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Insert:
					if (mnuAddNew.Enabled)
					{
						AddNew();
					}
					break;

				case Keys.Delete:
					if (mnuDelete.Enabled && (e.Modifiers == Keys.Shift))
					{
						Delete();
					}
					break;
			}
		}

		private void lvwList_KeyUp(object sender, KeyEventArgs e)
		{
			SetControls();
		}

		private void lvwList_MouseUp(object sender, MouseEventArgs e)
		{
			SetControls();
		}


		private void cboCity_SelectedIndexChanged(object sender, EventArgs e)
		{
			GrowCityCondition.GrowCity oGrowCity = SelectedGrowCity;
			if (oGrowCity != null)
			{
				if (oGrowCity.City != cboCity.SelectedItem)
				{
					oGrowCity.City = (ScenarioCity)cboCity.SelectedItem;
					SelectedItem.Text = oGrowCity.City.Name;

					if (oGrowCity.City == ScenarioCity.AnyCity)
					{
						oGrowCity.GrowQuantity = 0;
					}
				}
			}

			SetControls();
			SetNodeText();
		}

		private void cboInitialSize_SelectedIndexChanged(object sender, EventArgs e)
		{
			GrowCityCondition.GrowCity oGrowCity = SelectedGrowCity;
			if (oGrowCity != null)
			{
				if (oGrowCity.InitialSize != cboInitialSize.SelectedItem)
				{
					oGrowCity.InitialSize = (ScenarioCityType)cboInitialSize.SelectedItem;
					SelectedItem.SubItems[2].Text = oGrowCity.InitialSize.Name;
				}
			}

			SetControls();
			SetNodeText();
		}

		private void cboTargetSize_SelectedIndexChanged(object sender, EventArgs e)
		{
			GrowCityCondition.GrowCity oGrowCity = SelectedGrowCity;
			if (oGrowCity != null)
			{
				if (oGrowCity.TargetSize != cboTargetSize.SelectedItem)
				{
					oGrowCity.TargetSize = (ScenarioCityType)cboTargetSize.SelectedItem;
					SelectedItem.SubItems[3].Text = oGrowCity.TargetSize.Name;
				}
			}

			SetControls();
			SetNodeText();
		}

		private void txtCount_TextChanged(object sender, EventArgs e)
		{
			if (txtCount.Text != "")
			{
				GrowCityCondition.GrowCity oGrowCity = SelectedGrowCity;
				if (oGrowCity != null)
				{
					int nCount;
					if (int.TryParse(txtCount.Text, NumberStyles.Integer, CultureInfo.CurrentCulture, out nCount))
					{
						oGrowCity.GrowQuantity = nCount;
						SelectedItem.SubItems[1].Text = nCount.ToString();
						SetNodeText();
						return;
					}
				}
			}
		}

		private void txtCount_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if ((e.KeyChar < '0') || (e.KeyChar > '9'))
				{
					e.Handled = true;
				}
			}
		}


		private void mnuAddNew_Click(object sender, EventArgs e)
		{
			AddNew();
		}

		private void mnuDelete_Click(object sender, EventArgs e)
		{
			Delete();
		}

		private void AddNew()
		{
			GrowCityCondition.GrowCity oGrowCity = new GrowCityCondition.GrowCity((ScenarioSettings)m_oCondition.RootSetting);
			oGrowCity.GrowQuantity = 0;
			oGrowCity.City = (ScenarioCity)cboCity.Items[0];
			oGrowCity.InitialSize = ScenarioCityType.AnyCityType;
			oGrowCity.TargetSize = (ScenarioCityType)cboTargetSize.Items[cboTargetSize.Items.Count - 1];

			m_oCondition.Cities.Add(oGrowCity);
			ListViewItem lviNew = AddGrowCity(oGrowCity);

			lvwList.SelectedItems.Clear();
			lviNew.Selected = true;
			lvwList.FocusedItem = lviNew;

			LoadDetails();
			SetControls();
			SetNodeText();

			lvwList.Focus();
		}

		private void Delete()
		{
			int iIndex = lvwList.SelectedIndices[0];

			m_oCondition.Cities.RemoveAt(iIndex);
			lvwList.Items.RemoveAt(iIndex);

			if (iIndex >= lvwList.Items.Count) iIndex--;
			if (iIndex >= 0)
			{
				lvwList.Items[iIndex].Selected = true;
				lvwList.FocusedItem = lvwList.Items[iIndex];
			}

			SetControls();
			SetNodeText();

			lvwList.Focus();
		}
	}
}
