using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager.Editors
{
	public partial class ShipGoodsConditionEditor : ConditionEditorBase
	{
		private ShipGoodsCondition m_oCondition = null;


		public ShipGoodsConditionEditor()
		{
			InitializeComponent();
		}


		protected override void OnSetNode(object node)
		{
			base.OnSetNode(node);

			m_oCondition = (ShipGoodsCondition)node;

			lvwList.Items.Clear();
			for (int i = 0; i < m_oCondition.Goods.Count; i++)
			{
				ShipGoodsCondition.ShipGood oShipGood = m_oCondition.Goods[i];
				AddShipGood(oShipGood);
			}

			LoadDetails();
			SetControls();

			if (lvwList.Items.Count > 0)
			{
				lvwList.FocusedItem = lvwList.Items[0];
				lvwList.Items[0].Selected = true;
			}
		}

		private ListViewItem AddShipGood(ShipGoodsCondition.ShipGood shipGood)
		{
			ListViewItem lvi = new ListViewItem(shipGood.Good.Name);
			lvi.SubItems.Add(shipGood.Carloads.ToString());
			lvi.SubItems.Add(shipGood.FromCity.Name);
			lvi.SubItems.Add(shipGood.ToCity.Name);
			lvi.Tag = shipGood;

			lvwList.Items.Add(lvi);

			return lvi;
		}



		private void SetControls()
		{
			ShipGoodsCondition.ShipGood oShipGood = SelectedShipGood;
			if (oShipGood != null)
			{
				cboCityFrom.SelectedItem = oShipGood.FromCity;
				cboCityFrom.Enabled = true;

				cboCityTo.SelectedItem = oShipGood.ToCity;
				cboCityTo.Enabled = true;

				cboGood.SelectedItem = oShipGood.Good;
				cboGood.Enabled = true;

				txtCarloads.Text = oShipGood.Carloads.ToString();
				txtCarloads.Enabled = true;

				mnuAddNew.Enabled = true;
				mnuDelete.Enabled = true;
			}
			else
			{
				cboCityFrom.SelectedItem = null;
				cboCityFrom.Enabled = false;

				cboCityTo.SelectedItem = null;
				cboCityTo.Enabled = false;

				cboGood.SelectedItem = null;
				cboGood.Enabled = false;

				txtCarloads.Text = "";
				txtCarloads.Enabled = false;

				mnuAddNew.Enabled = true;
				mnuDelete.Enabled = false;
			}
		}

		private ShipGoodsCondition.ShipGood SelectedShipGood
		{
			get
			{
				ListViewItem lvi = SelectedItem;
				if (lvi == null) return null;

				return (ShipGoodsCondition.ShipGood)lvi.Tag;
			}
		}

		private ListViewItem SelectedItem
		{
			get { return (lvwList.SelectedItems.Count > 0) ? lvwList.SelectedItems[0] : null; }
		}


		private void LoadDetails()
		{
			Program.SuspendPaint(this);

			cboCityFrom.Items.Clear();
			cboCityTo.Items.Clear();
			cboCityFrom.Items.Add(ScenarioCity.AnyCity);
			cboCityTo.Items.Add(ScenarioCity.AnyCity);

			foreach (ScenarioCity oCity in SelectedScenario.Cities.Values)
			{
				cboCityFrom.Items.Add(oCity);
				cboCityTo.Items.Add(oCity);
			}

			cboGood.Items.Clear();
			foreach (ScenarioGood oGood in SelectedScenario.Goods.Values)
			{
				cboGood.Items.Add(oGood);
			}

			Program.ResumePaint(this);

		}


		private void lvwList_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwList.SelectedItems.Count > 0)
			{
				LoadDetails();
				SetControls();
			}
		}

		private void lvwList_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Insert:
					if (mnuAddNew.Enabled)
					{
						AddNew();
					}
					break;

				case Keys.Delete:
					if (mnuDelete.Enabled && (e.Modifiers == Keys.Shift))
					{
						Delete();
					}
					break;
			}
		}

		private void lvwList_KeyUp(object sender, KeyEventArgs e)
		{
			SetControls();
		}

		private void lvwList_MouseUp(object sender, MouseEventArgs e)
		{
			SetControls();
		}


		private void cboGood_SelectedIndexChanged(object sender, EventArgs e)
		{
			ShipGoodsCondition.ShipGood oShipGood = SelectedShipGood;
			if (oShipGood != null)
			{
				if (oShipGood.Good != cboGood.SelectedItem)
				{
					oShipGood.Good = (ScenarioGood)cboGood.SelectedItem;
					SelectedItem.Text = oShipGood.Good.Name;
				}
			}

			SetControls();
			SetNodeText();
		}

		private void cboCityFrom_SelectedIndexChanged(object sender, EventArgs e)
		{
			ShipGoodsCondition.ShipGood oShipGood = SelectedShipGood;
			if (oShipGood != null)
			{
				if (oShipGood.FromCity != cboCityFrom.SelectedItem)
				{
					oShipGood.FromCity = (ScenarioCity)cboCityFrom.SelectedItem;
					SelectedItem.SubItems[2].Text = oShipGood.FromCity.Name;
				}
			}

			SetControls();
			SetNodeText();
		}

		private void cboCityTo_SelectedIndexChanged(object sender, EventArgs e)
		{
			ShipGoodsCondition.ShipGood oShipGood = SelectedShipGood;
			if (oShipGood != null)
			{
				if (oShipGood.ToCity != cboCityTo.SelectedItem)
				{
					oShipGood.ToCity = (ScenarioCity)cboCityTo.SelectedItem;
					SelectedItem.SubItems[3].Text = oShipGood.ToCity.Name;
				}
			}

			SetControls();
			SetNodeText();
		}

		private void txtCarloads_TextChanged(object sender, EventArgs e)
		{
			if (txtCarloads.Text != "")
			{
				ShipGoodsCondition.ShipGood oShipGood = SelectedShipGood;
				if (oShipGood != null)
				{
					int nCarloads;
					if (int.TryParse(txtCarloads.Text, NumberStyles.Integer, CultureInfo.CurrentCulture, out nCarloads))
					{
						oShipGood.Carloads = nCarloads;
						SelectedItem.SubItems[1].Text = nCarloads.ToString();
						SetNodeText();
						return;
					}
				}
			}
		}

		private void txtCarloads_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if ((e.KeyChar < '0') || (e.KeyChar > '9'))
				{
					e.Handled = true;
				}
			}
		}


		private void mnuAddNew_Click(object sender, EventArgs e)
		{
			AddNew();
		}

		private void mnuDelete_Click(object sender, EventArgs e)
		{
			Delete();
		}

		private void AddNew()
		{
			ShipGoodsCondition.ShipGood oShipGood = new ShipGoodsCondition.ShipGood((ScenarioSettings)m_oCondition.RootSetting);
			oShipGood.Carloads = 10;
			oShipGood.FromCity = ScenarioCity.AnyCity;
			oShipGood.ToCity = ScenarioCity.AnyCity;
			oShipGood.Good = (ScenarioGood)cboGood.Items[0];

			m_oCondition.Goods.Add(oShipGood);
			ListViewItem lviNew = AddShipGood(oShipGood);

			lvwList.SelectedItems.Clear();
			lviNew.Selected = true;
			lvwList.FocusedItem = lviNew;

			LoadDetails();
			SetControls();
			SetNodeText();

			lvwList.Focus();
		}

		private void Delete()
		{
			int iIndex = lvwList.SelectedIndices[0];

			m_oCondition.Goods.RemoveAt(iIndex);
			lvwList.Items.RemoveAt(iIndex);

			if (iIndex >= lvwList.Items.Count) iIndex--;
			if (iIndex >= 0)
			{
				lvwList.Items[iIndex].Selected = true;
				lvwList.FocusedItem = lvwList.Items[iIndex];
			}

			SetControls();
			SetNodeText();

			lvwList.Focus();
		}
	}
}
