using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;


namespace Railroads.ScenarioManager.Editors
{
	public partial class SpeedRecordConditionEditor : ConditionEditorBase
	{
		private SpeedRecordCondition m_oCondition = null;


		public SpeedRecordConditionEditor()
		{
			InitializeComponent();
		}


		protected override void OnSetNode(object node)
		{
			base.OnSetNode(node);

			m_oCondition = (SpeedRecordCondition)node;

			cboFromCity.Items.Clear();
			cboFromCity.Items.Add(ScenarioCity.AnyCity);
			foreach (ScenarioCity oCity in SelectedScenario.Cities.Values)
			{
				cboFromCity.Items.Add(oCity);
			}
			cboFromCity.SelectedItem = m_oCondition.FromCity;

			cboToCity.Items.Clear();
			cboToCity.Items.Add(ScenarioCity.AnyCity);
			foreach (ScenarioCity oCity in SelectedScenario.Cities.Values)
			{
				cboToCity.Items.Add(oCity);
			}
			cboToCity.SelectedItem = m_oCondition.ToCity;

			cboGood.Items.Clear();
			cboGood.Items.Add(ScenarioGood.AnyGood);
			foreach (ScenarioGood oGood in SelectedScenario.Goods.Values)
			{
				cboGood.Items.Add(oGood);
			}
			cboGood.SelectedItem = m_oCondition.Good;

			cboTrain.Items.Clear();
			cboTrain.Items.Add(ScenarioTrain.AnyTrain);
			foreach (ScenarioTrain oTrain in SelectedScenario.Trains.Values)
			{
				cboTrain.Items.Add(oTrain);
			}
			cboTrain.SelectedItem = m_oCondition.Train;

			switch (m_oCondition.RecordType)
			{ 
				case SpeedRecordType.AnyRecord:
					txtSpeed.Text = "";
					break;

				case SpeedRecordType.SpecificSpeed:
					txtSpeed.Text = m_oCondition.Speed.ToString();
					break;
			}
		}

		private void cboFromCity_SelectedIndexChanged(object sender, EventArgs e)
		{
			m_oCondition.FromCity = (ScenarioCity)cboFromCity.SelectedItem;
			SetNodeText();
		}

		private void cboToCity_SelectedIndexChanged(object sender, EventArgs e)
		{
			m_oCondition.ToCity = (ScenarioCity)cboToCity.SelectedItem;
			SetNodeText();
		}

		private void cboTrain_SelectedIndexChanged(object sender, EventArgs e)
		{
			m_oCondition.Train = (ScenarioTrain)cboTrain.SelectedItem;
			SetNodeText();
		}

		private void cboGood_SelectedIndexChanged(object sender, EventArgs e)
		{
			m_oCondition.Good = (ScenarioGood)cboGood.SelectedItem;
			SetNodeText();
		}

		private void txtSpeed_TextChanged(object sender, EventArgs e)
		{
			if (txtSpeed.Text != "")
			{
				float fSpeed;
				if (float.TryParse(txtSpeed.Text, NumberStyles.Number, CultureInfo.CurrentCulture, out fSpeed))
				{
					m_oCondition.RecordType = SpeedRecordType.SpecificSpeed;
					m_oCondition.Speed = fSpeed;
					return;
				}
			}

			m_oCondition.RecordType = SpeedRecordType.AnyRecord;
		}

		private void txtSpeed_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if (((e.KeyChar < '0') || (e.KeyChar > '9')) && (e.KeyChar != '.') && (e.KeyChar != ','))
				{
					e.Handled = true;
				}
			}
		}
	}
}
