using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Railroads.ScenarioManager.Core;

namespace Railroads.ScenarioManager
{
	public partial class ExportForm : Form
	{
		public ExportForm()
		{
			InitializeComponent();
		}


		private void ExportForm_Load(object sender, EventArgs e)
		{
			cboScenarios.Sorted = false;

			/*
			foreach (KeyValuePair<string, Scenario> oPair in Manager.Instance.CoreScenarios)
			{
				cboScenarios.Items.Add(oPair.Value);
			}

			foreach (KeyValuePair<string, Scenario> oPair in Manager.Instance.UserScenarios)
			{
				cboScenarios.Items.Add(oPair.Value);
			}
			*/

			foreach (KeyValuePair<string, Scenario> oPair in Manager.Instance.LocalScenarios)
			{
				cboScenarios.Items.Add(oPair.Value);
			}

			cboScenarios.Sorted = true;
		}


		private void cmdBrowse_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(txtExportPath.Text))
			{
				saveFileDialog.FileName = cboScenarios.Text;
			}
			else
			{
				saveFileDialog.FileName = txtExportPath.Text;
			}

			if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
			{
				txtExportPath.Text = saveFileDialog.FileName;
			}
		}


		private void cmdOk_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(cboScenarios.Text))
			{
				MessageBox.Show("You must select a scenario to export.");
				return;
			}

			if (string.IsNullOrEmpty(txtExportPath.Text))
			{
				MessageBox.Show("You must provide an export path.");
				return;
			}

			Scenario oScenario = cboScenarios.SelectedItem as Scenario;

			if (oScenario.Type == ScenarioType.Local)
			{
				//Validate scenario
				List<string> oErrors = new List<string>();
				if (!oScenario.Validate(oErrors))
				{
					StringBuilder oBuilder = new StringBuilder();

					foreach (string oError in oErrors)
					{
						oBuilder.Append("\t");
						oBuilder.Append(oError);
						oBuilder.Append(Environment.NewLine);
					}

					MessageBox.Show("The selected scenario could not be exported because of the following errors:" + Environment.NewLine
					                + oBuilder.ToString(), "Validation Errors", MessageBoxButtons.OK, MessageBoxIcon.Error);

					return;
				}
			}

			using (new CursorChanger(Cursors.WaitCursor))
			{
				Manager.Instance.Export(oScenario, txtExportPath.Text, chkIncludeMaps.Checked);
				this.Close();
			}
		}
	}
}