using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Railroads.ScenarioManager.Core;

namespace Railroads.ScenarioManager
{
	public partial class MainForm
	{
		ScenarioCity _selectedCity;

		#region Event Handlers

		private void lvwCities_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwCities.SelectedItems.Count > 0)
			{
				using (new CursorChanger(Cursors.WaitCursor))
				{
					SaveCity();

					ScenarioCity city = lvwCities.SelectedItems[0].Tag as ScenarioCity;

					if (city != null)
					{
						_selectedCity = city;

						DisplayCity(city);
					}
				}
			}
		}

		private void cmdRemoveCity_Click(object sender, EventArgs e)
		{

		}

		private void cmdAddCity_Click(object sender, EventArgs e)
		{

		}

		#endregion

		#region Methods

		private void ClearCity()
		{
			txtCityName.Text = "";
			cboCityInitialType.Text = "";
			chkCityStartLocation.Checked = false;
			txtCityStartX.Text = "";
			txtCityStartY.Text = "";
			txtCityRotation.Text = "";
			cboCityTrackSide.Text = "";
		}

		private void LoadCities(ScenarioSettings settings)
		{
			ConditionForm.PopulateCityTypesCombo(cboCityInitialType, settings.Scenario);

			ListViewItem item;
			lvwCities.Items.Clear();

			foreach (KeyValuePair<string, ScenarioCity> pair in settings.Scenario.Cities)
			{
				item = new ListViewItem(pair.Value.Name);
				item.Tag = pair.Value;

				lvwCities.Items.Add(item);
			}

			if (lvwCities.Items.Count > 0)
			{
				lvwCities.Items[0].Selected = true;
			}
		}

		private void DisplayCity(ScenarioCity city)
		{
			txtCityName.Text = city.Name;
			cboCityInitialType.Text = city.Type;
			chkCityStartLocation.Checked = city.IsStartLocation;
			txtCityStartX.Text = city.StartX.ToString();
			txtCityStartY.Text = city.StartY.ToString();
			txtCityRotation.Text = city.Rotation.ToString();
			cboCityTrackSide.Text = city.DefaultTrackSide.ToString();
		}

		private void SaveCity()
		{
			if (_selectedCity == null) return;

			ScenarioCityType cityType = cboCityInitialType.SelectedItem as ScenarioCityType;
			if (cityType != null)
			{
				_selectedCity.Type = cityType.Name;
			}
			_selectedCity.IsStartLocation = chkCityStartLocation.Checked;
			
			int temp;
			if (int.TryParse(txtCityStartX.Text, out temp))
			{
				_selectedCity.StartX = temp;
			}
			if (int.TryParse(txtCityStartY.Text, out temp))
			{
				_selectedCity.StartY = temp;
			}

			double dTemp;
			if (double.TryParse(txtCityRotation.Text, out dTemp))
			{
				_selectedCity.Rotation = dTemp;
			}

			try
			{
				TrackSideType trackSide = (TrackSideType)Enum.Parse(typeof(TrackSideType), cboCityTrackSide.Text);
				_selectedCity.DefaultTrackSide = trackSide;
			}
			catch
			{
			}
		}

		private void AddCity()
		{
		}

		private void RemoveCity()
		{
		}

		#endregion
	}
}
