using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;

namespace Railroads.ScenarioManager
{
	public partial class MainForm
	{
		private bool m_bIgnoreEvents = false;


		private void lvwGoodsAvailable_SelectedIndexChanged(object sender, EventArgs e)
		{
			EnableGoodsControls();
		}

		private void lvwGoodsScenario_SelectedIndexChanged(object sender, EventArgs e)
		{
			SetMinMaxPrice();
			EnableGoodsControls();
		}

		private void lvwGoodsAvailable_DoubleClick(object sender, EventArgs e)
		{
			AddGoods();
		}

		private void lvwGoodsScenario_DoubleClick(object sender, EventArgs e)
		{
			RemoveGoods();
		}

		private void lvwGoodsAvailable_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Enter:
				case Keys.Insert:
					AddGoods();
					break;
			}
		}

		private void lvwGoodsScenario_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Enter:
				case Keys.Delete:
					RemoveGoods();
					break;
			}
		}

		private void txtMinimumPrice_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			ScenarioGood oGood = SelectedScenarioGood;
			if (oGood == null) return;

			int n = Parse(txtMinimumPrice.Text);
			if (oGood.MinPrice != n)
			{
				oGood.MinPrice = n;
				m_oSelectedScenario.IsDirty = true;

				SetGoodMinPrice(oGood, SelectedScenarioItem.SubItems[1], SelectedScenarioItem);
			}
		}

		private void txtMaximumPrice_TextChanged(object sender, EventArgs e)
		{
			if (m_bIgnoreEvents) return;

			ScenarioGood oGood = SelectedScenarioGood;
			if (oGood == null) return;

			int n = Parse(txtMaximumPrice.Text);
			if (oGood.MaxPrice != n)
			{
				oGood.MaxPrice = n;
				m_oSelectedScenario.IsDirty = true;

				SetGoodMaxPrice(oGood, SelectedScenarioItem.SubItems[2], SelectedScenarioItem);
			}
		}

		private void txtMinimumPrice_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar < ' ') return;
			if (((e.KeyChar >= '0') && (e.KeyChar <= '9')) || (e.KeyChar == '.') || (e.KeyChar == ',')) return;

			e.Handled = true;
		}

		private void txtMaximumPrice_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar < ' ') return;
			if (((e.KeyChar >= '0') && (e.KeyChar <= '9')) || (e.KeyChar == '.') || (e.KeyChar == ',')) return;

			e.Handled = true;
		}

		private void txtMinimumPrice_Leave(object sender, EventArgs e)
		{
			SetMinMaxPrice();
		}

		private void txtMaximumPrice_Leave(object sender, EventArgs e)
		{
			SetMinMaxPrice();
		}

		private void txtMinimumPrice_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
			{
				SetMinMaxPrice();
				txtMinimumPrice.SelectAll();
			}
		}

		private void txtMaximumPrice_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
			{
				SetMinMaxPrice();
				txtMaximumPrice.SelectAll();
			}
		}



		private void cmdGoodAdd_Click(object sender, EventArgs e)
		{
			AddGoods();
			EnableGoodsControls();
		}

		private void cmdGoodRemove_Click(object sender, EventArgs e)
		{
			RemoveGoods();
			EnableGoodsControls();
		}

		private void LoadGoods()
		{
			lvwGoodsAvailable.Sorting = SortOrder.None;
			lvwGoodsScenario.Sorting = SortOrder.None;

			lvwGoodsScenario.Items.Clear();
			foreach (ScenarioGood oScenarioGood in m_oSelectedScenario.Goods.Values)
			{
				AddScenarioGoodItem(oScenarioGood);
			}

			lvwGoodsAvailable.Items.Clear();
			foreach (Good oGood in ms_oManager.BaseGoods.Values)
			{
				if (!m_oSelectedScenario.Goods.ContainsKey(oGood.Name))
				{
					AddAvailableGoodItem(oGood);
				}
			}

			lvwGoodsAvailable.Sorting = SortOrder.Ascending;
			lvwGoodsScenario.Sorting = SortOrder.Ascending;

			EnableGoodsControls();
		}

		private void AddGoods()
		{
			while (lvwGoodsAvailable.SelectedItems.Count > 0)
			{
				ListViewItem lvi = lvwGoodsAvailable.SelectedItems[0];

				Good oGood = lvi.Tag as Good;

				ScenarioGood oScenarioGood = ScenarioGood.FromGood(oGood);
				oScenarioGood.Name = oGood.Name;

				m_oSelectedScenario.Goods.Add(oScenarioGood.Name, oScenarioGood);
				m_oSelectedScenario.IsDirty = true;

				AddScenarioGoodItem(oScenarioGood);
				lvi.Remove();
			}
		}

		private void RemoveGoods()
		{
			while (lvwGoodsScenario.SelectedItems.Count > 0)
			{
				ListViewItem lvi = lvwGoodsScenario.SelectedItems[0];

				ScenarioGood oGood = lvi.Tag as ScenarioGood;

				m_oSelectedScenario.Goods.Remove(oGood.Name);
				m_oSelectedScenario.IsDirty = true;

				AddAvailableGoodItem(GetBaseGood(oGood));
				lvi.Remove();
			}
		}

		public void CheckGood(params Good[] goods)
		{
			bool bUpdated = false;

			foreach (Good oGood in goods)
			{
				if (!m_oSelectedScenario.Goods.ContainsKey(oGood.Name))
				{
					m_oSelectedScenario.Goods.Add(oGood.Name, ScenarioGood.FromGood(oGood));
					bUpdated = true;
				}
			}

			if (bUpdated)
			{
				LoadGoods();
			}
		}

		private void AddScenarioGoodItem(ScenarioGood good)
		{
			ListViewItem lvi = new ListViewItem(good.Name);
			SetGoodMinPrice(good, lvi.SubItems.Add(""), lvi);
			SetGoodMaxPrice(good, lvi.SubItems.Add(""), lvi);
			lvi.Tag = good;

			lvwGoodsScenario.Items.Add(lvi);
		}

		private void SetGoodMinPrice(ScenarioGood good, ListViewItem.ListViewSubItem subItem, ListViewItem item)
		{
			if (good.MinPrice == -1)
			{
				Good oBaseGood = GetBaseGood(good);
				subItem.Text = ((oBaseGood != null) ? ToString(oBaseGood.MinPrice) : "");
				subItem.ForeColor = SystemColors.ControlDark;
				subItem.Font = new Font(lvwGoodsScenario.Font, FontStyle.Italic);
				item.UseItemStyleForSubItems = false;
			}
			else 
			{
				subItem.Text = ToString(good.MinPrice);
				subItem.Font = lvwGoodsScenario.Font;
				subItem.ForeColor = SystemColors.WindowText;
			}
		}

		private void SetGoodMaxPrice(ScenarioGood good, ListViewItem.ListViewSubItem subItem, ListViewItem item)
		{
			if (good.MaxPrice == -1)
			{
				Good oBaseGood = GetBaseGood(good);
				subItem.Text = ((oBaseGood != null) ? ToString(oBaseGood.MaxPrice) : "");
				subItem.ForeColor = SystemColors.ControlDark;
				subItem.Font = new Font(lvwGoodsScenario.Font, FontStyle.Italic);
				item.UseItemStyleForSubItems = false;
			}
			else
			{
				subItem.Text = ToString(good.MaxPrice);
				subItem.Font = lvwGoodsScenario.Font;
				subItem.ForeColor = SystemColors.WindowText;
			}
		}

		private Good GetBaseGood(ScenarioGood good)
		{
			Good oBaseGood = null;
			ms_oManager.BaseGoods.TryGetValue(good.Name, out oBaseGood);
			return oBaseGood;
		}

		private void AddAvailableGoodItem(Good good)
		{
			ListViewItem lvi = new ListViewItem(good.Name);
			lvi.SubItems.Add(ToString(good.MinPrice));
			lvi.SubItems.Add(ToString(good.MaxPrice));
			lvi.Tag = good;

			lvwGoodsAvailable.Items.Add(lvi);
		}

		private ScenarioGood SelectedScenarioGood
		{
			get 
			{
				if (lvwGoodsScenario.FocusedItem == null) return null;
				return lvwGoodsScenario.FocusedItem.Tag as ScenarioGood;
			}
		}

		private ListViewItem SelectedScenarioItem
		{
			get { return lvwGoodsScenario.FocusedItem; }
		}

		private void SetMinMaxPrice()
		{
			m_bIgnoreEvents = true;

			ScenarioGood oGood = SelectedScenarioGood;
			if (oGood != null)
			{
				txtMinimumPrice.Text = ToString(oGood.MinPrice);
				txtMaximumPrice.Text = ToString(oGood.MaxPrice);
			}
			else 
			{
				txtMinimumPrice.Text = "";
				txtMaximumPrice.Text = "";
			}

			m_bIgnoreEvents = false;
		}

		private void EnableGoodsControls()
		{ 
			cmdGoodAdd.Enabled = (lvwGoodsAvailable.SelectedItems.Count > 0);
			cmdGoodRemove.Enabled = (lvwGoodsScenario.SelectedItems.Count > 0);

			txtMinimumPrice.Enabled = (lvwGoodsScenario.FocusedItem != null);
			txtMaximumPrice.Enabled = (lvwGoodsScenario.FocusedItem != null);
		}

		private string ToString(int value)
		{
			if (value == -1) return "";
			return value.ToString("N0");
		}

		private int Parse(string text)
		{
			int n = -1;

			int.TryParse(text, NumberStyles.Number, CultureInfo.CurrentCulture, out n);
			
			if (n == 0) n = -1;

			return n;
		}
	}
}
