using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Railroads.ScenarioManager.Core;

namespace Railroads.ScenarioManager
{
	public partial class MainForm
	{
		private void cmdAddTrain_Click(object sender, EventArgs e)
		{
			AddTrains();
		}

		private void cmdRemoveTrain_Click(object sender, EventArgs e)
		{
			RemoveTrains();
		}


		private void lvwTrainsAvailable_SelectedIndexChanged(object sender, EventArgs e)
		{
			SetControls();
		}

		private void lvwTrainsScenario_SelectedIndexChanged(object sender, EventArgs e)
		{
			SetControls();
		}


		private void lvwTrainsAvailable_DoubleClick(object sender, EventArgs e)
		{
			AddTrains();
		}

		private void lvwTrainsAvailable_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{ 
				case Keys.Enter:
				case Keys.Insert:
					AddTrains();
					break;
			}
		}

		private void lvwTrainsScenario_DoubleClick(object sender, EventArgs e)
		{
			RemoveTrains();
		}

		private void lvwTrainsScenario_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Enter:
				case Keys.Delete:
					RemoveTrains();
					break;
			}
		}


		private void SetControls()
		{
			cmdAddTrain.Enabled = (lvwTrainsAvailable.SelectedItems.Count > 0);
			cmdRemoveTrain.Enabled = (lvwTrainsScenario.SelectedItems.Count > 0);
		}

		private void LoadTrains()
		{
			lvwTrainsAvailable.Sorting = SortOrder.None;
			lvwTrainsScenario.Sorting = SortOrder.None;

			lvwTrainsScenario.Items.Clear();
			foreach (KeyValuePair<string, ScenarioTrain> oPair in m_oSelectedScenario.Trains)
			{
				AddScenarioTrainItem(oPair.Value);
			}

			lvwTrainsAvailable.Items.Clear();
			foreach (KeyValuePair<string, Train> oPair in ms_oManager.BaseTrains)
			{
				if (!m_oSelectedScenario.Trains.ContainsKey(oPair.Value.Tag))
				{
					AddAvailableTrainItem(oPair.Value);
				}
			}

			lvwTrainsAvailable.Sorting = SortOrder.Ascending;
			lvwTrainsScenario.Sorting = SortOrder.Ascending;

			cohAvailableTrains.Width = -2;
			cohScenarioTrains.Width = -2;
		}

		private void AddTrains()
		{
			while (lvwTrainsAvailable.SelectedItems.Count > 0)
			{
				ListViewItem lvi = lvwTrainsAvailable.SelectedItems[0];
				Train oTrain = lvi.Tag as Train;

				ScenarioTrain oScenarioTrain = new ScenarioTrain(oTrain.Tag, oTrain.Name);

				m_oSelectedScenario.Trains.Add(oScenarioTrain.Tag, oScenarioTrain);
				m_oSelectedScenario.IsDirty = true;

				AddScenarioTrainItem(oScenarioTrain);
				lvi.Remove();
			}
		}

		private void RemoveTrains()
		{
			while (lvwTrainsScenario.SelectedItems.Count > 0)
			{
				ListViewItem lvi = lvwTrainsScenario.SelectedItems[0];
				Train oTrain = lvi.Tag as Train;

				m_oSelectedScenario.Trains.Remove(oTrain.Tag);
				m_oSelectedScenario.IsDirty = true;

				AddAvailableTrainItem(oTrain);
				lvi.Remove();
			}
		}

		private void AddScenarioTrainItem(ScenarioTrain train)
		{
			ListViewItem lvi = new ListViewItem(train.Name);
			lvi.Tag = train;

			lvwTrainsScenario.Items.Add(lvi);
		}

		private void AddAvailableTrainItem(Train train)
		{
			ListViewItem lvi = new ListViewItem(train.Name);
			lvi.Tag = train;

			lvwTrainsAvailable.Items.Add(lvi);
		}
	}
}
