using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Railroads.ScenarioManager.Core;
using Railroads.ScenarioManager.Editors;

namespace Railroads.ScenarioManager
{
	public partial class MainForm : Form
	{
		private static readonly Manager		ms_oManager = Manager.Instance;

		
		private Scenario					m_oSelectedScenario;

		private ListViewGroup				m_lvgInProgressGroup;
		private ListViewGroup				m_lvgUserScenariosGroup;
		private ListViewGroup				m_lvgScenariosGroup;
		private ListViewGroup				m_lvgScenariosError;



		public MainForm()
		{
			InitializeComponent();

			m_lvgInProgressGroup = new ListViewGroup("Work In Progress");
			m_lvgUserScenariosGroup = new ListViewGroup("User Scenarios");
			m_lvgScenariosGroup = new ListViewGroup("Game Scenarios");
			m_lvgScenariosError = new ListViewGroup("Errors");

			lvwScenarios.Groups.Add(m_lvgInProgressGroup);
			lvwScenarios.Groups.Add(m_lvgUserScenariosGroup);
			lvwScenarios.Groups.Add(m_lvgScenariosGroup);
			lvwScenarios.Groups.Add(m_lvgScenariosError);

			EnableTabs(false, false);
		}

		public Scenario SelectedScenario
		{
			get { return m_oSelectedScenario; }
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			LoadScenarios();
			cityManagerControl.LoadCombos();
		}

		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			try
			{
				SaveScenario();
			}
			catch
			{
			}
		}

		private void lvwScenarios_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lvwScenarios.SelectedItems.Count > 0)
			{
				using (new CursorChanger(Cursors.WaitCursor))
				{
					//SaveScenario();

					m_oSelectedScenario = CurrentScenario;

					if ((m_oSelectedScenario != null) && !m_oSelectedScenario.HasError && (m_oSelectedScenario.State != ScenarioState.Loaded))
					{
						m_oSelectedScenario.Load();
					}

					DisplayScenario(m_oSelectedScenario);
				}
			}
			else 
			{
				ClearScenario();
				EnableTabs(false, false);
			}

			EnableControls();
		}

		private void txtScenarioName_TextChanged(object sender, EventArgs e)
		{
			if ((lvwScenarios.SelectedItems.Count > 0) && (m_oSelectedScenario != null))
			{
				if ((m_oSelectedScenario.Type == ScenarioType.Local) && (m_oSelectedScenario == CurrentScenario))
				{
					lvwScenarios.SelectedItems[0].Text = txtScenarioName.Text;
				}
			}
		}

		private void cmdAddScenario_Click(object sender, EventArgs e)
		{
			using (new CursorChanger(Cursors.WaitCursor))
			{
				AddScenario();
			}
		}

		private void cmdRemoveScenario_Click(object sender, EventArgs e)
		{
			if (lvwScenarios.SelectedItems.Count > 0)
			{
				using (new CursorChanger(Cursors.WaitCursor))
				{
					RemoveScenario(lvwScenarios.SelectedItems[0]);
				}
			}
		}

		private void txtTurnsPerMonth_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar >= ' ')
			{
				if ((e.KeyChar < '0') || (e.KeyChar > '9'))
				{
					e.Handled = true;
				}
			}
		}

		private void cmdShallowColor_Click(object sender, EventArgs e)
		{
			colorDialog1.Color = cmdShallowColor.BackColor;

			if (colorDialog1.ShowDialog(this) == DialogResult.OK)
			{
				cmdShallowColor.BackColor = colorDialog1.Color;
				cmdShallowColor.Tag = colorDialog1.Color;
			}
		}

		private void cmdDeepColor_Click(object sender, EventArgs e)
		{
			colorDialog1.Color = cmdDeepColor.BackColor;

			if (colorDialog1.ShowDialog(this) == DialogResult.OK)
			{
				cmdDeepColor.BackColor = colorDialog1.Color;
				cmdDeepColor.Tag = colorDialog1.Color;
			}
		}

		private void cmdCreateScenario_Click(object sender, EventArgs e)
		{
			if (lvwScenarios.SelectedItems.Count > 0)
			{
				Scenario oScenario = CurrentScenario;

				if (oScenario != null)
				{
					CreateScenario(oScenario);
				}
			}
		}

		private void cmdSave_Click(object sender, EventArgs e)
		{
			//Save any changes
			SaveScenario();

			ms_oManager.Save();

			MessageBox.Show("Successfully saved all changes to workspace.");

			if (m_oSelectedScenario != null)
			{
				if (m_oSelectedScenario.State != ScenarioState.Loaded)
				{
					m_oSelectedScenario.Load();
				}

				DisplayScenario(m_oSelectedScenario);
			}
		}

		private void cmdReload_Click(object sender, EventArgs e)
		{
			if (MessageBox.Show(this, "Are you sure you want to reload?  Any changes made since your last save will be lost.",
					"Confirm Reload", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
			{
				using (new CursorChanger(Cursors.WaitCursor))
				{
					ReloadScenarios();
				}
			}
		}

		private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using (AboutBox frm = new AboutBox())
			{
				frm.ShowDialog(this);
			}
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void exportToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using (ExportForm frm = new ExportForm())
			{
				SaveScenario();

				frm.ShowDialog(this);
			}
		}

		private void importToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using (ImportForm frm = new ImportForm())
			{
				SaveScenario();

				if (frm.ShowDialog(this) == DialogResult.OK)
				{
					m_oSelectedScenario = null;
					cityManagerControl.SelectedScenario = null;
					industryEditor.SelectedScenario = null;
					cityTypeEditor.SelectedScenario = null;

					ClearScenario();

					//Reload form data
					LoadScenarios();
				}
			}
		}

		private void optionsToolStripMenuItem1_Click(object sender, EventArgs e)
		{
			using (OptionsForm frm = new OptionsForm())
			{
				SaveScenario();

				if (frm.ShowDialog(this) == DialogResult.OK)
				{
					ReloadScenarios();
				}
			}
		}


		private void ClearScenario()
		{
			txtScenarioName.Text = "";
			txtSubtitle.Text = "";
			txtAuthor.Text = "";
			txtDescription.Text = "";
			txtTurnsPerMonth.Text = "";

			txtMapThumbnail.Text = "";
			txtMapFile.Text = "";

			cityManagerControl.SelectedScenario = null;
			industryEditor.SelectedScenario = null;
			cityTypeEditor.SelectedScenario = null;
		}

		private void LoadScenarios()
		{
			lvwScenarios.Items.Clear();

			foreach (KeyValuePair<string, Scenario> oPair in ms_oManager.CoreScenarios)
			{
				AddScenarioItem(m_lvgScenariosGroup, oPair.Value);
			}

			foreach (KeyValuePair<string, Scenario> oPair in ms_oManager.UserScenarios)
			{
				AddScenarioItem(m_lvgUserScenariosGroup, oPair.Value);
			}

			foreach (KeyValuePair<string, Scenario> oPair in ms_oManager.LocalScenarios)
			{
				AddScenarioItem(m_lvgInProgressGroup, oPair.Value);
			}

			EnableControls();
		}

		private ListViewItem AddScenarioItem(ListViewGroup group, Scenario scenario)
		{
			foreach (ListViewItem lvi2 in lvwScenarios.Items)
			{
				if (lvi2.Group != group) continue;
				if (lvi2.Text == scenario.ToString())
				{
					//already exists, replace the objects
					lvi2.Tag = scenario;
					return lvi2;
				}
			}


			ListViewItem lvi = new ListViewItem(scenario.ToString());
			if (scenario.HasError)
			{
				//lvi.Font = new Font(lvi.Font, FontStyle.Italic);
				lvi.ForeColor = Color.Red;
				lvi.Group = m_lvgScenariosError;
			}
			else
			{
				lvi.SubItems.Add(scenario.ScenarioSettings.Author);
				lvi.SubItems.Add(scenario.Type.ToString());
				lvi.Group = group;

			}

			lvi.Tag = scenario;

			lvwScenarios.Items.Add(lvi);

			return lvi;
		}

		private void DisplayScenario(Scenario scenario)
		{
			string sProblems = "";
			foreach (string s in scenario.Problems)
			{
				if (sProblems != "") sProblems += Environment.NewLine + Environment.NewLine;
				sProblems += s;
			}

			txtProblems.Text = sProblems;


			if (scenario.HasError)
			{
				m_oSelectedScenario = null;

				ClearScenario();

				Exception ex;
				if (ms_oManager.ScenarioErrors.TryGetValue(scenario.Tag, out ex))
				{
					txtScenarioErrors.Text = "An error occured while loading the scenario: ";
					txtScenarioErrors.Text += Environment.NewLine;
					txtScenarioErrors.Text += Environment.NewLine;

					while (ex.InnerException != null)
					{
						txtScenarioErrors.Text += ex.Message;
						txtScenarioErrors.Text += Environment.NewLine;
						txtScenarioErrors.Text += Environment.NewLine;

						ex = ex.InnerException;
					}

					txtScenarioErrors.Text += ex.Message;
					txtScenarioErrors.Text += Environment.NewLine;
					txtScenarioErrors.Text += Environment.NewLine;

					txtScenarioErrors.Text += ex.StackTrace;

					tabControl.SelectedTab = tabErrors;
				}

				EnableTabs(false, true);
			}
			else
			{
				txtScenarioErrors.Text = "There were no errors.";

				ScenarioSettings oSettings = scenario.ScenarioSettings;

				if (scenario.BaseScenario == null)
				{
					txtBaseScenario.Text = "<Standalone>";
					txtScenarioName.ReadOnly = true;
				}
				else
				{
					txtBaseScenario.Text = scenario.BaseScenario.ToString();
					txtScenarioName.ReadOnly = false;
				}

				txtScenarioName.Text = oSettings.ScenarioName;
				txtSubtitle.Text = oSettings.Subtitle;
				txtAuthor.Text = oSettings.Author;
				txtDescription.Text = oSettings.Description;
				txtTurnsPerMonth.Text = oSettings.TurnsPerMonth.ToString();

				chkMultiplayer.Checked = oSettings.IsMultiplayer;

				cmdDeepColor.BackColor = oSettings.DeepColor;
				cmdDeepColor.Tag = oSettings.DeepColor;

				cmdShallowColor.BackColor = oSettings.ShallowColor;
				cmdShallowColor.Tag = oSettings.ShallowColor;

				txtMapThumbnail.Text = oSettings.MapThumbnail;
				txtMapFile.Text = oSettings.MapFile;

				cityManagerControl.SelectedScenario = m_oSelectedScenario;
				industryEditor.SelectedScenario = m_oSelectedScenario;
				cityTypeEditor.SelectedScenario = m_oSelectedScenario;
				LoadGoods();
				LoadTrains();

				LoadConditions(oSettings);

				EnableTabs(true, true);

				if (tabControl.SelectedTab == tabErrors)
				{
					tabControl.SelectedTab = tabGeneral;
				}

				if (txtProblems.Text != "")
				{
					tabControl.SelectedTab = tabErrors;
				}
			}
		}

		private void EnableTabs(bool enableAll, bool enableErrors)
		{
			foreach (TabPage oTabPage in tabControl.TabPages)
			{
				oTabPage.Enabled = enableAll;
			}
			tabErrors.Enabled = enableErrors;
		}

		private void SaveScenario()
		{
			if (m_oSelectedScenario == null)
			{
				return;
			}

			ScenarioSettings oSettings = m_oSelectedScenario.ScenarioSettings;

			oSettings.ScenarioName = txtScenarioName.Text;
			oSettings.Subtitle = txtSubtitle.Text;
			oSettings.Author = txtAuthor.Text;
			oSettings.Description = txtDescription.Text;
			oSettings.TurnsPerMonth = int.Parse(txtTurnsPerMonth.Text);

			oSettings.IsMultiplayer = chkMultiplayer.Checked;

			oSettings.DeepColor = (Color)cmdDeepColor.Tag;
			oSettings.ShallowColor = (Color)cmdShallowColor.Tag;

			oSettings.MapThumbnail = txtMapThumbnail.Text;
			oSettings.MapFile = txtMapFile.Text;

			cityManagerControl.SaveCity();

			CopyScenario();
		}

		private void CopyScenario()
		{
			if ((m_oSelectedScenario.Type != ScenarioType.Local)/* && m_oSelectedScenario.IsDirty*/)
			{
				m_oSelectedScenario.IsDirty = false;

				Scenario oScenario = (Scenario)m_oSelectedScenario.Clone();
				oScenario.ScenarioSettings.ScenarioName = oScenario.ScenarioSettings.ScenarioName;
				oScenario.ScenarioSettings.Author = oScenario.ScenarioSettings.Author;
				oScenario.IsDirty = true;

				ms_oManager.LocalScenarios.Add(oScenario.Tag, oScenario);

				//Reload the real scenario
				m_oSelectedScenario.Load();

				AddScenarioItem(m_lvgInProgressGroup, oScenario);
			}
		}

		private void AddScenario()
		{
			SaveScenario();

			using (NewScenarioForm frm = new NewScenarioForm())
			{
				if (frm.ShowDialog(this) == DialogResult.OK)
				{
					Scenario oScenario = frm.Scenario;

					if (oScenario != null)
					{
						ms_oManager.LocalScenarios.Add(oScenario.Tag, oScenario);

						ListViewItem lvi = AddScenarioItem(m_lvgInProgressGroup, oScenario);
						lvi.Selected = true;
					}
				}
			}
		}

		private void RemoveScenario(ListViewItem item)
		{
			Scenario oScenario = item.Tag as Scenario;

			if (oScenario != null)
			{
				if (item.Group == m_lvgScenariosGroup)
				{
					MessageBox.Show("You cannot delete Firaxis scenarios.");

					//You cannot delete firaxis scenarios
					return;
				}

				if (MessageBox.Show("Are you sure you want to delete the selected scenario?", "Confirm Deletion",
									MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
				{
					return;
				}

				ms_oManager.RemoveScenario(oScenario);

				oScenario.Delete();

				if (m_oSelectedScenario == oScenario)
				{
					m_oSelectedScenario = null;
				}

				item.Remove();

				if (lvwScenarios.Items.Count > 0)
				{
					lvwScenarios.Items[0].Selected = true;
				}
				else
				{
					ClearScenario();
				}
			}
		}

		private void CreateScenario(Scenario scenario)
		{
			if (scenario.Type != ScenarioType.Local)
			{
				return;
			}

			//Make sure scenario is saved
			SaveScenario();

			//Make sure all changes have been written to the workspace
			ms_oManager.Save();

			//Validate scenario
			List<string> oErrors = new List<string>();
			if (!scenario.Validate(oErrors))
			{
				StringBuilder oSB = new StringBuilder();

				foreach (string sError in oErrors)
				{
					oSB.Append("\t");
					oSB.Append(sError);
					oSB.Append(Environment.NewLine);
				}

				MessageBox.Show(this, "The selected scenario could not be created because of the following errors:" + Environment.NewLine
								+ oSB.ToString(), "Validation Errors", MessageBoxButtons.OK, MessageBoxIcon.Error);

				return;
			}

			ScenarioExistsActionType oAction = ScenarioExistsActionType.CreateNew;

			if (ms_oManager.ScenarioExists(scenario))
			{
				using (ScenarioExistsForm frm = new ScenarioExistsForm())
				{
					if (frm.ShowDialog(this) == DialogResult.Cancel)
					{
						return;
					}
					else
					{
						oAction = frm.Action;
					}
				}
			}

			using (new CursorChanger(Cursors.WaitCursor))
			{
				//Now finally create the scenario
				ms_oManager.Create(scenario, oAction);

				MessageBox.Show("The '" + scenario.ScenarioSettings.ScenarioName +
				                "' scenario has been created in your My Documents Railroads! scenarios directory.");

				ReloadScenarios();
			}
		}

		private void ReloadScenarios()
		{
			m_oSelectedScenario = null;
			cityManagerControl.SelectedScenario = null;

			ClearScenario();

			//Reload all data
			ms_oManager.Load();

			//Reload form data
			LoadScenarios();
		}


		private void btnClose_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private Scenario CurrentScenario
		{
			get { return (lvwScenarios.SelectedItems.Count > 0) ? lvwScenarios.SelectedItems[0].Tag as Scenario : null; }
		}

		private void EnableControls()
		{
			cmdAddScenario.Enabled = true;

			Scenario oScenario = CurrentScenario;

			bool bEnableRemove = (oScenario != null);
			if (bEnableRemove && (lvwScenarios.SelectedItems[0].Group == m_lvgScenariosGroup))
			{
				bEnableRemove = false;
			}
			cmdRemoveScenario.Enabled = bEnableRemove;
			cmdSave.Enabled = (oScenario != null);
		}


		private void LoadConditions(ScenarioSettings scenario)
		{
			tvwEras.BeginUpdate();

			tvwEras.Nodes.Clear();

			for (int iEra = 0; iEra < scenario.Eras.Count; iEra++)
			{ 
				Era oEra = scenario.Eras[iEra];

				TreeNode tnEra = new TreeNode(oEra.EraName, 0, 0);
				tnEra.NodeFont = new Font(tvwEras.Font, FontStyle.Bold);
				tnEra.Tag = oEra;
				tvwEras.Nodes.Add(tnEra);

				for (int iCondition = 0; iCondition < oEra.Conditions.Count; iCondition++)
				{
					Condition oCondition = oEra.Conditions[iCondition];

					TreeNode tnCondition = new TreeNode(oCondition.ToString(), 1, 1);
					tnCondition.Tag = oCondition;
					tnEra.Nodes.Add(tnCondition);
				}

				tnEra.Expand();
			}

			tvwEras.EndUpdate();

			if (tvwEras.Nodes.Count > 0)
			{
				tvwEras.SelectedNode = tvwEras.Nodes[0];
			}
		}

		private void tvwEras_AfterSelect(object sender, TreeViewEventArgs e)
		{
			Program.SuspendPaint(this);

			EditorBase oEditor = EditorBase.GetEditor(e.Node);
			if (oEditor != null)
			{
				oEditor.SetNode(e.Node, this);

				oEditor.Dock = DockStyle.Fill;

				if (splConditions.Panel2.Controls.Count > 0)
				{
					if (oEditor != splConditions.Panel2.Controls[0])
					{
						splConditions.Panel2.Controls.Add(oEditor);
						splConditions.Panel2.Controls.RemoveAt(0);
					}
				}
				else 
				{
					splConditions.Panel2.Controls.Add(oEditor);
				}
			}
			else 
			{
				//remove existing editor
				splConditions.Panel2.Controls.Clear();
			}

			Program.ResumePaint(this);
		}



		private void mnuNewEra_Click(object sender, EventArgs e)
		{
			Era oEra = new Era(m_oSelectedScenario.ScenarioSettings);
			oEra.EraName = "New Era";

			TreeNode tn = new TreeNode(oEra.EraName, 0, 0);
			tn.NodeFont = new Font(tvwEras.Font, FontStyle.Bold);
			tn.Tag = oEra;

			m_oSelectedScenario.ScenarioSettings.Eras.Add(oEra);

			tvwEras.Nodes.Add(tn);
			tvwEras.SelectedNode = tn;
		}


		private void mnuBuildStuff_Click(object sender, EventArgs e)
		{
			AddCondition(new BuildStuffCondition(SelectedScenario.ScenarioSettings));
		}

		private void mnuConnect_Click(object sender, EventArgs e)
		{
			AddCondition(new ConnectCondition(SelectedScenario.ScenarioSettings));
		}

		private void mnuGrowCity_Click(object sender, EventArgs e)
		{
			AddCondition(new GrowCityCondition(SelectedScenario.ScenarioSettings));
		}

		private void mnuNetWorth_Click(object sender, EventArgs e)
		{
			AddCondition(new NetWorthCondition(SelectedScenario.ScenarioSettings));
		}

		private void mnuOwnStock_Click(object sender, EventArgs e)
		{
			AddCondition(new OwnStockCondition(SelectedScenario.ScenarioSettings));
		}

		private void mnuOwnStuff_Click(object sender, EventArgs e)
		{
			AddCondition(new OwnStuffCondition(SelectedScenario.ScenarioSettings));
		}

		private void mnuShipGoods_Click(object sender, EventArgs e)
		{
			AddCondition(new ShipGoodsCondition(SelectedScenario.ScenarioSettings));
		}

		private void mnuSpeedRecord_Click(object sender, EventArgs e)
		{
			AddCondition(new SpeedRecordCondition(SelectedScenario.ScenarioSettings));
		}


		private void AddCondition(Condition condition)
		{
			TreeNode tn = tvwEras.SelectedNode;
			if (tn.Parent != null) tn = tn.Parent;

			Era oEra = tn.Tag as Era;

			oEra.Conditions.Add(condition);


			TreeNode tnNew = new TreeNode(condition.ToString(), 1, 1);
			tnNew.Tag = condition;
			tn.Nodes.Add(tnNew);

			tvwEras.SelectedNode = tnNew;
		}


		private void mnuMoveUp_Click(object sender, EventArgs e)
		{
			DoMoveUp();
		}

		private void mnuMoveDown_Click(object sender, EventArgs e)
		{
			DoMoveDown();
		}

		private void mnuDelete_Click(object sender, EventArgs e)
		{
			DoDelete();
		}


		private void DoMoveUp()
		{
			TreeNode tn = tvwEras.SelectedNode;
			if (tn != null)
			{
				tvwEras.BeginUpdate();

				if (tn.Parent == null)
				{
					//era node
					Era oEra = tn.Tag as Era;

					List<Era> oEras = m_oSelectedScenario.ScenarioSettings.Eras;
					int i = oEras.IndexOf(oEra);
					Debug.Assert(i == tn.Index);

					oEras.RemoveAt(i);
					oEras.Insert(i - 1, oEra);

					tvwEras.Nodes.RemoveAt(i);
					tvwEras.Nodes.Insert(i - 1, tn);
				}
				else
				{
					//condition node
					Condition oCondition = tn.Tag as Condition;
					Era oEra = tn.Parent.Tag as Era;
					List<Condition> oConditions = oEra.Conditions;
					int i = oConditions.IndexOf(oCondition);
					Debug.Assert(i == tn.Index);

					if (tn.PrevNode != null)
					{
						//moving inside the same era
						TreeNodeCollection oNodes = tn.Parent.Nodes;
						oNodes.RemoveAt(i);
						oConditions.RemoveAt(i);

						oNodes.Insert(i - 1, tn);
						oConditions.Insert(i - 1, oCondition);
					}
					else 
					{
						//moving to the previous era
						TreeNodeCollection oNodes = tn.Parent.Nodes;
						TreeNode oTargetEraNode = tn.Parent.PrevNode;
						oNodes.RemoveAt(i);
						oConditions.RemoveAt(i);

						Era oTargetEra = oTargetEraNode.Tag as Era;
						oTargetEraNode.Nodes.Add(tn);
						oTargetEra.Conditions.Add(oCondition);
					}
				}

				tvwEras.EndUpdate();

				tvwEras.SelectedNode = tn;
			}

		}

		private void DoMoveDown()
		{
			TreeNode tn = tvwEras.SelectedNode;
			if (tn != null)
			{
				tvwEras.BeginUpdate();

				if (tn.Parent == null)
				{
					//era node
					Era oEra = tn.Tag as Era;

					List<Era> oEras = m_oSelectedScenario.ScenarioSettings.Eras;
					int i = oEras.IndexOf(oEra);
					Debug.Assert(i == tn.Index);

					oEras.RemoveAt(i);
					oEras.Insert(i + 1, oEra);

					tvwEras.Nodes.RemoveAt(i);
					tvwEras.Nodes.Insert(i + 1, tn);
				}
				else
				{
					//condition node
					Condition oCondition = tn.Tag as Condition;
					Era oEra = tn.Parent.Tag as Era;

					List<Condition> oConditions = oEra.Conditions;
					int i = oConditions.IndexOf(oCondition);
					Debug.Assert(i == tn.Index);

					if (tn.NextNode != null)
					{
						//moving inside the same era
						oConditions.RemoveAt(i);
						oConditions.Insert(i + 1, oCondition);

						TreeNodeCollection oNodes = tn.Parent.Nodes;
						oNodes.RemoveAt(i);
						oNodes.Insert(i + 1, tn);
					}
					else 
					{
						//moving to the next era
						TreeNodeCollection oNodes = tn.Parent.Nodes;
						TreeNode oTargetEraNode = tn.Parent.NextNode;
						oNodes.RemoveAt(i);
						oConditions.RemoveAt(i);

						Era oTargetEra = oTargetEraNode.Tag as Era;
						oTargetEraNode.Nodes.Insert(0, tn);
						oTargetEra.Conditions.Insert(0, oCondition);
					}

				}

				tvwEras.EndUpdate();

				tvwEras.SelectedNode = tn;
			}			
		}

		private void DoDelete()
		{
			TreeNode tn = tvwEras.SelectedNode;
			if (tn != null)
			{
				tvwEras.BeginUpdate();

				if (tn.Parent == null)
				{
					//era node
					Era oEra = tn.Tag as Era;
					m_oSelectedScenario.ScenarioSettings.Eras.Remove(oEra);
				}
				else 
				{
					//condition node
					Condition oCondition = tn.Tag as Condition;
					Era oEra = tn.Parent.Tag as Era;

					oEra.Conditions.Remove(oCondition);
				}

				TreeNode tnToSelect = tn.NextNode;
				if (tnToSelect == null) tnToSelect = tn.PrevNode;
				if (tnToSelect == null) tnToSelect = tn.Parent;

				tn.Remove();

				tvwEras.EndUpdate();

				tvwEras.SelectedNode = tnToSelect;
			}			
		}

		private void tvwEras_MouseDown(object sender, MouseEventArgs e)
		{
			TreeViewHitTestInfo oHitTest = tvwEras.HitTest(e.Location);
			if ((oHitTest != null) && (oHitTest.Node != null))
			{
				tvwEras.SelectedNode = oHitTest.Node;
			}
		}

		private void ctxEras_Opening(object sender, CancelEventArgs e)
		{
			TreeNode tn = tvwEras.SelectedNode;
			if (tn != null)
			{
				object o = tn.Tag;

				mnuNewEra.Enabled = (tn.Parent == null);
				mnuNewCondition.Enabled = true;
			}
			else 
			{
				mnuNewEra.Enabled = true;
				mnuNewCondition.Enabled = false;
			}

			mnuMoveUp.Enabled = CanMoveUp();
			mnuMoveDown.Enabled = CanMoveDown();
			mnuDelete.Enabled = CanDelete();

			Debug.WriteLine("Node:" + tn.Text);
		}

		private void tvwEras_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Delete:
					if (e.Modifiers == Keys.Shift)
					{
						if (CanDelete())
						{
							DoDelete();
							e.Handled = true;
						}
					}
					break;

				case Keys.Up:
					if ((e.Modifiers == Keys.Shift) || (e.Modifiers == Keys.Control))
					{
						if (CanMoveUp())
						{
							DoMoveUp();
							e.Handled = true;
						}
					}
					break;

				case Keys.Down:
					if ((e.Modifiers == Keys.Shift) || (e.Modifiers == Keys.Control))
					{
						if (CanMoveDown())
						{
							DoMoveDown();
							e.Handled = true;
						}
					}
					break;
			}
		}


		private bool CanMoveUp()
		{
			TreeNode tn = tvwEras.SelectedNode;
			if (tn == null) return false;
			if (tn.PrevNode != null) return true;

			if (tn.Parent != null)
			{
				if (tn.Parent.PrevNode != null) return true;
			}

			return false;
		}

		private bool CanMoveDown()
		{
			TreeNode tn = tvwEras.SelectedNode;
			if (tn == null) return false;
			if (tn.NextNode != null) return true;

			if (tn.Parent != null)
			{
				if (tn.Parent.NextNode != null) return true;
			}

			return false;
		}

		private bool CanDelete()
		{
			TreeNode tn = tvwEras.SelectedNode;
			if (tn == null) return false;
			return (tn.Nodes.Count == 0);
		}

		private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (m_oSelectedScenario == null) return;

			if (tabControl.SelectedTab == tabVictoryConditions)
			{
				LoadConditions(m_oSelectedScenario.ScenarioSettings);
			}
			else if (tabControl.SelectedTab == tabIndustry)
			{
				industryEditor.SelectedScenario = m_oSelectedScenario;
			}
			else if (tabControl.SelectedTab == tabCityTypes)
			{
				cityTypeEditor.SelectedScenario = m_oSelectedScenario;
			}
		}
	}
}