using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Forms;
using Railroads.ScenarioManager.Core;

namespace Railroads.ScenarioManager
{
	public partial class NewScenarioForm : Form
	{
		public NewScenarioForm()
		{
			InitializeComponent();
		}

		private Scenario _scenario;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public Scenario Scenario
		{
			[DebuggerStepThrough]
			get { return _scenario; }
		}

		private void cmdOk_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(cboBaseScenario.Text))
			{
				MessageBox.Show("You must define a base scenario type.");

				return;
			}

			if (string.IsNullOrEmpty(txtScenarioName.Text))
			{
				MessageBox.Show("You must define a scenario name.");

				return;
			}

			bool isNameUnique = true;
			foreach (KeyValuePair<string, Scenario> pair in Manager.Instance.LocalScenarios)
			{
				if (pair.Value.ScenarioSettings.ScenarioName == txtScenarioName.Text)
				{
					isNameUnique = false;

					break;
				}
			}
			if (!isNameUnique)
			{
				MessageBox.Show("A scenario with this name already exists.  Please change it and try again.");

				return;
			}

			_scenario = (Scenario) ((Scenario) cboBaseScenario.SelectedItem).Clone();
			_scenario.Tag = Guid.NewGuid().ToString();
			_scenario.ScenarioSettings.ScenarioName = txtScenarioName.Text;
			_scenario.ScenarioSettings.Author = txtAuthor.Text;

			this.DialogResult = DialogResult.OK;
			this.Close();
		}

		private void NewConditionForm_Load(object sender, EventArgs e)
		{
			List<Scenario> scenarioList = new List<Scenario>();

			//Add core scenarios to combo box
			foreach (KeyValuePair<string, Scenario> pair in Manager.Instance.CoreScenarios)
			{
				scenarioList.Add(pair.Value);
			}

			//Add my scenarios to combo box
			foreach (KeyValuePair<string, Scenario> pair in Manager.Instance.UserScenarios)
			{
				scenarioList.Add(pair.Value);
			}

			scenarioList.Sort(new Comparison<Scenario>(
			                  	delegate(Scenario s1, Scenario s2) {
			                  		return s1.ToString().CompareTo(s2.ToString());
			                  	})
				);

			foreach (Scenario scenario in scenarioList)
			{
				cboBaseScenario.Items.Add(scenario);
			}

			cboBaseScenario.SelectedIndex = 0;
			txtScenarioName.Text = "My New Scenario " + (Manager.Instance.LocalScenarios.Count + 1).ToString();
		}
	}
}