using System;
using System.IO;
using System.Windows.Forms;
using Railroads.ScenarioManager.Core;

namespace Railroads.ScenarioManager
{
	public partial class OptionsForm : Form
	{
		public OptionsForm()
		{
			InitializeComponent();
		}

		ApplicationSettings _settings;

		private void OptionsForm_Load(object sender, EventArgs e)
		{
			_settings = ManagerApplication.AppSettings;

			cboLocale.Text = _settings.Locale;

			if (!string.IsNullOrEmpty(_settings.RailroadsDirectory))
			{
				txtRailroadsFolder.Text = Utility.AddTrailingSlash(_settings.RailroadsDirectory);
			}

			if (string.IsNullOrEmpty(_settings.MyRailroadsDirectory))
			{
				txtMyRailroadsFolder.Text = Locations.Default_MyGamesRailroadsFolder;
			}
			else
			{
				txtMyRailroadsFolder.Text = Utility.AddTrailingSlash(_settings.MyRailroadsDirectory);
			}
			
			if (!string.IsNullOrEmpty(_settings.WorkingDirectory))
			{
				txtWorkspaceFolder.Text = Utility.AddTrailingSlash(_settings.WorkingDirectory);
			}


			//fill in the details
			if (txtMyRailroadsFolder.Text == "")
			{ 

			}
		}

		private void cmdOk_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(cboLocale.Text))
			{
				MessageBox.Show(this, "You must define a locale.");
				return;
			}

			//Check railroads folder
			if (string.IsNullOrEmpty(txtRailroadsFolder.Text))
			{
				MessageBox.Show(this, "You must define the Sid Meier's Railroads! folder.");
				return;
			}

			txtRailroadsFolder.Text = Utility.AddTrailingSlash(txtRailroadsFolder.Text);
			if (!Directory.Exists(txtRailroadsFolder.Text))
			{
				MessageBox.Show(this, "Could not find the Railroads! folder.  Please check the path and try again.");
				return;
			}

			try
			{
				Manager.CheckRailroadsDirectory(txtRailroadsFolder.Text);
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
				return;
			}

			//Check my games railroads folder
			if (string.IsNullOrEmpty(txtMyRailroadsFolder.Text))
			{
				MessageBox.Show(this, "You must define your My Games Railroads! folder.");
				return;
			}

			txtMyRailroadsFolder.Text = Utility.AddTrailingSlash(txtMyRailroadsFolder.Text);
			if (!Directory.Exists(txtMyRailroadsFolder.Text))
			{
				MessageBox.Show(this, "Could not find the My Games Railroads! folder.  Please check the path and try again.");
				return;
			}

			//Check workspace directory
			if (string.IsNullOrEmpty(txtWorkspaceFolder.Text))
			{
				MessageBox.Show(this, "You must define a workspace folder.");

				return;
			}

			txtWorkspaceFolder.Text = Utility.AddTrailingSlash(txtWorkspaceFolder.Text);
			if (!Directory.Exists(txtWorkspaceFolder.Text))
			{
				Directory.CreateDirectory(txtWorkspaceFolder.Text);
			}

			_settings.Locale = cboLocale.Text;
			_settings.RailroadsDirectory = txtRailroadsFolder.Text;
			_settings.WorkingDirectory = txtWorkspaceFolder.Text;
			_settings.MyRailroadsDirectory = txtMyRailroadsFolder.Text;

			ManagerApplication.Configuration.Save();

			DialogResult = DialogResult.OK;
			Close();
		}

		private void cmdBrowseRailroads_Click(object sender, EventArgs e)
		{
			folderBrowserDialog1.ShowNewFolderButton = false;
			folderBrowserDialog1.Description = "Select the path where Sid Meier's Railroads! is installed.";

			folderBrowserDialog1.SelectedPath = txtRailroadsFolder.Text;

			if (folderBrowserDialog1.ShowDialog(this) == DialogResult.OK)
			{
				txtRailroadsFolder.Text = Utility.AddTrailingSlash(folderBrowserDialog1.SelectedPath);
			}
		}

		private void cmdBrowseWorkspace_Click(object sender, EventArgs e)
		{
			folderBrowserDialog1.ShowNewFolderButton = true;
			folderBrowserDialog1.Description =
				"Select the path where scenarions that you are working on will be stored.  " +
				"This should a subdirectory of where the Scenario Manager is installed.";

			folderBrowserDialog1.SelectedPath = txtWorkspaceFolder.Text;

			if (folderBrowserDialog1.ShowDialog(this) == DialogResult.OK)
			{
				txtWorkspaceFolder.Text = Utility.AddTrailingSlash(folderBrowserDialog1.SelectedPath);
			}
		}

		private void cmdBrowseMyRailroads_Click(object sender, EventArgs e)
		{
			folderBrowserDialog1.ShowNewFolderButton = true;
			folderBrowserDialog1.Description =
				"Select the folder where the Railroads! user documents are stored.  " +
				@"This is usually in <My Documents>\My Games\Sid Meier's Railroads!\" +
				"folder but can be different if you've installed Railroads! into an non-standard location.";

			folderBrowserDialog1.SelectedPath = txtMyRailroadsFolder.Text;

			if (folderBrowserDialog1.ShowDialog(this) == DialogResult.OK)
			{
				txtMyRailroadsFolder.Text = Utility.AddTrailingSlash(folderBrowserDialog1.SelectedPath);
			}
		}

		private void cmdCancel_Click(object sender, EventArgs e)
		{
			this.Close();
		}
	}
}