using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class Industry : XmlSetting
	{
		private List<Resource>		m_oResources = new List<Resource>();
		private List<string>		m_oDisplayNames = new List<string>();
		private string				m_sName;
		private string				m_sModel;
		private string				m_sGrowKfm;
		private string				m_sTerrainMap;
		private string				m_sSoundScape;
		private string				m_sAuctionImage;
		private string				m_sReportImage;
		private string				m_sIconFile;
		private float				m_fScale;
		private int					m_nCost;
		private bool				m_bIsCostal;
		private bool				m_bIsMountainous;
		private bool				m_bIsInCity;



		public class Resource : XmlSetting
		{
			internal Good		m_oInput;
			internal Good		m_oOutput;
			internal float		m_oInputOutputRatio;

			
			
			public Resource()
			{
			}

			public Resource(ScenarioSettings rootSetting)
			{
				base.RootSetting = rootSetting;
			}


			protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

			public Good Input { get { return m_oInput; } set { Scenario.SetProperty(ref m_oInput, value); } }
			public Good Output { get { return m_oOutput; } set { Scenario.SetProperty(ref m_oOutput, value); } }
			public float InputOutputRatio { get { return m_oInputOutputRatio; } set { Scenario.SetProperty(ref m_oInputOutputRatio, value); } }


			public override void ToXml(XmlNode node)
			{
				ToXml_Create(node);
			}


			protected XmlNode ToXml_Create(XmlNode node)
			{
				XmlNode oResourceNode = Utility.CreateChildElement(node, "Resource");

				Utility.SetChildElementValue(oResourceNode, "Input", m_oInput.XmlName);
				Utility.SetChildElementValue(oResourceNode, "Output", m_oOutput.XmlName);
				Utility.SetChildElementValue(oResourceNode, "InputOutputRatio", m_oInputOutputRatio);

				return oResourceNode;
			}

			public override void FromXml(Scenario scenario, XmlNode node)
			{
				string sInput = Utility.GetChildElementValue<string>(node, "Input");
				string sOutput = Utility.GetChildElementValue<string>(node, "Output");

				if (RootSetting == null)
				{
					m_oInput = Manager.Instance.GetBaseGood(sInput);
					m_oOutput = Manager.Instance.GetBaseGood(sOutput);
				}
				else
				{
					m_oInput = Condition.GetGood(this.Scenario, sInput);
					m_oOutput = Condition.GetGood(this.Scenario, sOutput);
				}

				m_oInputOutputRatio = Utility.GetChildElementValue<float>(node, "InputOutputRatio");
			}


			public override object Clone()
			{
				Resource oClone = new Resource();

				oClone.m_oInput = (Good)m_oInput.Clone();
				oClone.m_oOutput = (Good)m_oOutput.Clone();
				oClone.m_oInputOutputRatio = m_oInputOutputRatio;

				return oClone;
			}


			public override bool Validate(List<string> errors)
			{
				return true;
			}


			public void SetRootSetting(XmlSetting rootSetting)
			{
				base.RootSetting = rootSetting;
			}
		}


		protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

		public List<Resource> Resources { get { return m_oResources; } }
		public List<string> DisplayNames { get { return m_oDisplayNames; } }


		public string Name { get { return m_sName; } set { Scenario.SetProperty(ref m_sName, value); } }
		public string Model { get { return m_sModel; } set { Scenario.SetProperty(ref m_sModel, value); } }
		public string GrowKfm { get { return m_sGrowKfm; } set { Scenario.SetProperty(ref m_sGrowKfm, value); } }
		public string TerrainMap { get { return m_sTerrainMap; } set { Scenario.SetProperty(ref m_sTerrainMap, value); } }
		public string SoundScape { get { return m_sSoundScape; } set { Scenario.SetProperty(ref m_sSoundScape, value); } }
		public string AuctionImage { get { return m_sAuctionImage; } set { Scenario.SetProperty(ref m_sAuctionImage, value); } }
		public string ReportImage { get { return m_sReportImage; } set { Scenario.SetProperty(ref m_sReportImage, value); } }
		public string IconFile { get { return m_sIconFile; } set { Scenario.SetProperty(ref m_sIconFile, value); } }
		public float Scale { get { return m_fScale; } set { Scenario.SetProperty(ref m_fScale, value); } }
		public int Cost { get { return m_nCost; } set { Scenario.SetProperty(ref m_nCost, value); } }
		public bool IsCostal { get { return m_bIsCostal; } set { Scenario.SetProperty(ref m_bIsCostal, value); } }
		public bool IsMountainous { get { return m_bIsMountainous; } set { Scenario.SetProperty(ref m_bIsMountainous, value); } }
		public bool IsInCity { get { return m_bIsInCity; } set { Scenario.SetProperty(ref m_bIsInCity, value); } }


		public override void ToXml(XmlNode node)
		{
			ToXml(node, this, false);
		}

		public XmlNode ToXml(XmlNode node, Industry industry, bool onlyChanged)
		{
			XmlNode oIndustryNode = node.SelectSingleNode("RRTIndustry[szName='" + m_sName + "']");
			if (oIndustryNode == null)
			{
				oIndustryNode = Utility.CreateChildElement(node, "RRTIndustry");
			}
			else
			{
				oIndustryNode.RemoveAll();
			}

			Utility.SetChildElementValue(oIndustryNode, "szName", m_sName);

			bool bShortCut = ((industry == this) || !onlyChanged);
			if (bShortCut || (industry.m_sModel != m_sModel))
			{
				Utility.SetChildElementValue(oIndustryNode, "szModel", m_sModel);
			}

			if (bShortCut || (industry.m_sGrowKfm != m_sGrowKfm))
			{
				Utility.SetChildElementValue(oIndustryNode, "szGrowKFM", m_sGrowKfm);
			}

			if (bShortCut || (industry.m_sTerrainMap != m_sTerrainMap))
			{
				if (!string.IsNullOrEmpty(m_sTerrainMap)) Utility.SetChildElementValue(oIndustryNode, "szTerrainmap", m_sTerrainMap);
			}

			if (bShortCut || (industry.m_fScale != m_fScale))
			{
				Utility.SetChildElementValue(oIndustryNode, "fScale", m_fScale);
			}

			if (bShortCut || (industry.m_bIsCostal != m_bIsCostal))
			{
				Utility.SetChildElementValue(oIndustryNode, "bIsCoastal", (m_bIsCostal ? 1 : 0));
			}

			if (bShortCut || (industry.m_bIsMountainous != m_bIsMountainous))
			{
				Utility.SetChildElementValue(oIndustryNode, "bIsMountainous", (m_bIsMountainous ? 1 : 0));
			}

			if (bShortCut || (industry.m_bIsInCity != m_bIsInCity))
			{
				Utility.SetChildElementValue(oIndustryNode, "bInCity", (m_bIsInCity ? 1 : 0));
			}

			if (bShortCut || (industry.m_nCost != m_nCost))
			{
				Utility.SetChildElementValue(oIndustryNode, "iCost", m_nCost);
			}

			if ((m_oResources != null) && (m_oResources.Count > 0))
			{
				Utility.RemoveElements(oIndustryNode, "Production");

				XmlNode oProductionNode = Utility.CreateChildElement(oIndustryNode, "Production");
				foreach (Resource oResource in m_oResources)
				{
					oResource.ToXml(oProductionNode);
				}
			}

			if (bShortCut || (industry.m_sSoundScape != m_sSoundScape))
			{
				if (!string.IsNullOrEmpty(m_sSoundScape)) Utility.SetChildElementValue(oIndustryNode, "szSoundScape", m_sSoundScape);
			}

			if (bShortCut || (industry.m_sAuctionImage != m_sAuctionImage))
			{
				Utility.SetChildElementValue(oIndustryNode, "szAuctionImage", m_sAuctionImage);
			}

			if (bShortCut || (industry.m_sReportImage != m_sReportImage))
			{
				Utility.SetChildElementValue(oIndustryNode, "szReportImage", m_sReportImage);
			}

			if (bShortCut || (industry.m_sIconFile != m_sIconFile))
			{
				XmlNode oIconNode = Utility.CreateChildElement(oIndustryNode, "szIcon");
				Utility.SetChildElementValue(oIconNode, "szFile", m_sIconFile);
			}

			if ((m_oDisplayNames != null) && (m_oDisplayNames.Count > 0))
			{
				XmlNode oNamesNode = Utility.CreateChildElement(oIndustryNode, "DisplayNames");

				foreach (string sName in m_oDisplayNames)
				{
					XmlNode oNameNode = Utility.CreateChildElement(oNamesNode, "szDisplayName");
					oNameNode.InnerText = sName;
				}
			}

			return oIndustryNode;
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			m_sName = Utility.GetChildElementValue<string>(node, "szName");
			m_sModel = Utility.GetChildElementValue<string>(node, "szModel", m_sModel, false);
			m_sGrowKfm = Utility.GetChildElementValue<string>(node, "szGrowKFM", m_sGrowKfm, false);
			m_sTerrainMap = Utility.GetChildElementValue<string>(node, "szTerrainmap", m_sTerrainMap, false);
			m_sSoundScape = Utility.GetChildElementValue<string>(node, "szSoundScape", m_sTerrainMap, false);
			m_sAuctionImage = Utility.GetChildElementValue<string>(node, "szAuctionImage", m_sAuctionImage, false);
			m_sReportImage = Utility.GetChildElementValue<string>(node, "szReportImage", m_sReportImage, false);
			m_sIconFile = Utility.GetChildElementValue<string>(node, "szIcon/szFile", m_sIconFile, false);

			m_fScale = Utility.GetChildElementValue<float>(node, "fScale", m_fScale, false);
			m_nCost = Utility.GetChildElementValue<int>(node, "iCost", m_nCost, false);

			m_bIsCostal = (Utility.GetChildElementValue<int>(node, "bIsCoastal", (m_bIsCostal ? 1 : 0), false) == 1);
			m_bIsMountainous = (Utility.GetChildElementValue<int>(node, "bIsMountainous", (m_bIsMountainous ? 1 : 0), false) == 1);
			m_bIsInCity = (Utility.GetChildElementValue<int>(node, "bInCity", (m_bIsInCity ? 1 : 0), false) == 1);

			m_oResources = base.LoadSubSetting<Resource>(node, "Production/Resource");

			m_oDisplayNames = new List<string>();
			XmlNodeList oList = node.SelectNodes("DisplayNames/szDisplayName");
			if ((oList != null) && (oList.Count > 0))
			{
				foreach (XmlNode oDisplayNode in oList)
				{
					m_oDisplayNames.Add(oDisplayNode.InnerText);
				}
			}
		}

		public override string ToString()
		{
			return m_sName;
		}

		public override object Clone()
		{
			Industry oClone = new Industry();
			Copy(oClone);
			return oClone;
		}

		public void CopyToScenarioIndusty(ScenarioIndustry scenarioIndustry)
		{
			Copy(scenarioIndustry);
		}

		public override bool Validate(List<string> errors)
		{
			return true;
		}

		private void Copy(Industry copy)
		{
			copy.m_sName = m_sName;
			copy.m_sAuctionImage = m_sAuctionImage;
			copy.m_nCost = m_nCost;
			copy.m_sGrowKfm = m_sGrowKfm;
			copy.m_sIconFile = m_sIconFile;
			copy.m_bIsCostal = m_bIsCostal;
			copy.m_bIsInCity = m_bIsInCity;
			copy.m_bIsMountainous = m_bIsMountainous;
			copy.m_sModel = m_sModel;
			copy.m_sReportImage = m_sReportImage;
			copy.m_fScale = m_fScale;
			copy.m_sTerrainMap = m_sTerrainMap;
			copy.m_sSoundScape = m_sSoundScape;

			copy.m_oResources = new List<Resource>();
			foreach (Resource oResource in m_oResources)
			{
				copy.m_oResources.Add((Resource)oResource.Clone());
			}

			copy.m_oDisplayNames = new List<string>();
			foreach (string sName in m_oDisplayNames)
			{
				copy.m_oDisplayNames.Add(sName);
			}
		}
	}
}