using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public abstract class TaggedItem : XmlSetting
	{
		private string			m_sTag;
		private string			m_sName;
		private string			m_sGender = "M";


		public string Tag { get { return m_sTag; } set { m_sTag = value; } }
		public string Name { get { return m_sName; } set { m_sName = value; } }
		public string Gender { get { return m_sGender; } set { m_sGender = value; } }


		public override void ToXml(XmlNode node)
		{
			XmlNode oTextNode = Utility.GetOrCreateChildElement(node, "TextKey[Tag='" + m_sTag + "']", "TextKey");

			Utility.SetChildElementValue(oTextNode, "Tag", m_sTag);
			Utility.SetChildElementValue(oTextNode, "Text", m_sName);
			Utility.SetChildElementValue(oTextNode, "Gender", m_sGender);
		}

		public override void FromXml(Scenario scenario, XmlNode node)
		{
			m_sTag = Utility.GetChildElementValue<string>(node, "Tag");
			m_sName = Utility.GetChildElementValue<string>(node, "Text");
			m_sGender = Utility.GetChildElementValue<string>(node, "Gender", "M", false);
		}


		public override string ToString()
		{
			return m_sName;
		}

		public override object Clone()
		{
			Train oClone = new Train();
			oClone.m_sTag = m_sTag;
			oClone.m_sName = m_sName;
			oClone.m_sGender = m_sGender;
			return oClone;
		}

		public override bool Validate(List<string> errors)
		{
			return true;
		}
	}
}