using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class ConnectCondition : Condition
	{
		private List<ScenarioCity> m_oCities = new List<ScenarioCity>();


		public ConnectCondition() { }
		public ConnectCondition(ScenarioSettings rootSetting) : base(rootSetting) { }


		protected override ConditionType OnGetConditionType() { return ConditionType.ConnectCities; }


		public List<ScenarioCity> Cities { get { return m_oCities; } }


		protected override void ToXml_Internal(XmlNode node)
		{
			XmlNode oCondNode = Utility.CreateChildElement(node, "ConnectionList");

			foreach (ScenarioCity city in m_oCities)
			{
				XmlNode oCityNode = Utility.CreateChildElement(oCondNode, "szCity");
				oCityNode.InnerText = city.Tag;
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.FromXml(scenario, node);

			m_oCities = new List<ScenarioCity>();

			XmlNodeList oCityNodes = node.SelectNodes("ConnectionList/szCity");
			if ((oCityNodes != null) && (oCityNodes.Count > 0))
			{
				foreach (XmlNode oCityNode in oCityNodes)
				{
					string sCityTag = oCityNode.InnerText;
					ScenarioCity city = GetCity(scenario, sCityTag);
					m_oCities.Add(city);
				}
			}
		}


		public override bool Validate(List<string> errors)
		{
			if (m_oCities.Count == 0)
			{
				errors.Add("You must define at least one city to connect.");
				return false;
			}

			return true;
		}


		protected override Condition Clone_Internal()
		{
			ConnectCondition oClone = new ConnectCondition();

			oClone.m_oCities = new List<ScenarioCity>();
			foreach (ScenarioCity oCity in m_oCities)
			{
				oClone.m_oCities.Add((ScenarioCity) oCity.Clone());
			}

			return oClone;
		}


		protected override string DefaultSuccessBitmap { get { return "event_connectcity.dds"; } }


		public override string ToString()
		{
			string s = "";
			string sSeparator = ", ";

			for (int i = 0; i < m_oCities.Count; i++)
			{
				ScenarioCity oCity = m_oCities[i];

				if (i == m_oCities.Count - 1) sSeparator = " and ";

				if (s != "") s += sSeparator;
				s += oCity.ToString();
			}

			return "Connect " + s;
		}
	}
}