using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class NetWorthCondition : Condition
	{
		private long		m_nNetWorth;


		public NetWorthCondition() { }
		public NetWorthCondition(ScenarioSettings rootSetting) : base(rootSetting) { }


		protected override ConditionType OnGetConditionType() { return ConditionType.NetWorth; }


		public long NetWorth { get { return m_nNetWorth; } set { Scenario.SetProperty(ref m_nNetWorth, value); } }


		protected override void ToXml_Internal(XmlNode node)
		{
			Utility.SetChildElementValue(node, "iThreshold", m_nNetWorth);
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.FromXml(scenario, node);
			m_nNetWorth = Utility.GetChildElementValue<long>(node, "iThreshold");
		}


		public override bool Validate(List<string> errors)
		{
			if (m_nNetWorth < 1)
			{
				errors.Add("The net worth must be greater than zero.");
				return false;
			}

			return true;
		}


		protected override Condition Clone_Internal()
		{
			NetWorthCondition oClone = new NetWorthCondition();
			oClone.m_nNetWorth = m_nNetWorth;
			return oClone;
		}


		protected override string DefaultSuccessBitmap { get { return "event_networth.dds"; } }


		public override string ToString()
		{
			return "Achieve net worth of $" + m_nNetWorth.ToString("###,###,##0"); ;
		}
	}
}