using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class OwnStuffCondition : Condition
	{
		private List<OwnThing>		m_oOwnThings = new List<OwnThing>();


		public OwnStuffCondition() { }
		public OwnStuffCondition(ScenarioSettings rootSetting) : base(rootSetting) { }


		protected override ConditionType OnGetConditionType() { return ConditionType.OwnStuff; }


		public class OwnThing : XmlSetting
		{
			private string			m_sObjectType = "Industry";
			private Industry		m_oIndustryType;
			private ScenarioCity	m_oCity;


			internal OwnThing()
			{
			}

			public OwnThing(ScenarioSettings rootSetting)
			{
				base.RootSetting = rootSetting;
			}

			protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

			public string ObjectType { get { return m_sObjectType; } set { m_sObjectType = value; } }


			public Industry IndustryType { get { return m_oIndustryType; } set { Scenario.SetProperty(ref m_oIndustryType, value); } }
			public ScenarioCity City { get { return m_oCity; } set { Scenario.SetProperty(ref m_oCity, value); } }


			public override void ToXml(XmlNode node)
			{
				XmlNode oCondNode = Utility.CreateChildElement(node, "OwnListItem");

				Utility.SetChildElementValue(oCondNode, "szObjectType", m_sObjectType);
				Utility.SetChildElementValue(oCondNode, "szObjectName", m_oIndustryType.Name);
				Utility.SetChildElementValue(oCondNode, "szCityLocation", m_oCity.Tag);
			}


			public override void FromXml(Scenario scenario, XmlNode node)
			{
				ScenarioSettings oRoot = this.RootSetting as ScenarioSettings;
				if (oRoot == null)
				{
					throw new ArgumentException("Could not find parent scenario settings instance.");
				}

				m_sObjectType = Utility.GetChildElementValue<string>(node, "szObjectType");
				string sTag = Utility.GetChildElementValue<string>(node, "szObjectName");
				m_oIndustryType = GetIndustry(oRoot.Scenario, sTag);

				sTag = Utility.GetChildElementValue<string>(node, "szCityLocation");
				m_oCity = GetCity(oRoot.Scenario, sTag);
			}


			public override object Clone()
			{
				OwnThing oClone = new OwnThing();

				oClone.m_sObjectType = m_sObjectType;
				oClone.m_oCity = (ScenarioCity) m_oCity.Clone();
				oClone.m_oIndustryType = (Industry) m_oIndustryType.Clone();

				return oClone;
			}


			public override bool Validate(List<string> errors)
			{
				return true;
			}
		}



		public List<OwnThing> OwnThings { get { return m_oOwnThings; } }


		internal override void SetRootSetting(XmlSetting root)
		{
			base.SetRootSetting(root);
			foreach (OwnThing oThing in m_oOwnThings)
			{
				oThing.RootSetting = root;
			}
		}


		protected override void ToXml_Internal(XmlNode node)
		{
			XmlNode oCondNode = Utility.CreateChildElement(node, "OwnList");
			foreach (OwnThing thing in m_oOwnThings)
			{
				thing.ToXml(oCondNode);
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.FromXml(scenario, node);

			OwnThing thing = null;
			m_oOwnThings = new List<OwnThing>();

			XmlNodeList oNodes = node.SelectNodes("OwnList/OwnListItem");
			if ((oNodes != null) && (oNodes.Count > 0))
			{
				foreach (XmlNode oThingNode in oNodes)
				{
					thing = new OwnThing();
					thing.RootSetting = this.RootSetting;

					thing.FromXml(scenario, oThingNode);

					m_oOwnThings.Add(thing);
				}
			}
		}


		public override bool Validate(List<string> errors)
		{
			if (m_oOwnThings.Count == 0)
			{
				errors.Add("You must define at least one thing to own.");
				return false;
			}

			return true;
		}


		protected override Condition Clone_Internal()
		{
			OwnStuffCondition oClone = new OwnStuffCondition();

			oClone.m_oOwnThings = new List<OwnThing>();
			foreach (OwnThing oThing in m_oOwnThings)
			{
				oClone.m_oOwnThings.Add((OwnThing) oThing.Clone());
			}

			return oClone;
		}


		protected override string DefaultSuccessBitmap { get { return "event_buildstuff.dds"; } }


		public override string ToString()
		{
			string s = "";

			foreach (OwnThing oThing in m_oOwnThings)
			{
				if (s != "") s += ", ";
				s += oThing.IndustryType.Name + " in " + oThing.City.ToString();
			}

			return "Build or buy " + s;
		}
	}
}