using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Drawing;

using System.IO;
using System.Text;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class Scenario : ICloneable
	{
		public const string ANY_CITY = "ANY_CITY";
		public const string ANY_CITY_TYPE = "ANY_CITY_TYPE";
		public const string ANY_TRAIN = "ANY_TRAIN";
		public const string ANY_GOOD = "ANY_GOOD";
		public const string NO_GOOD = "None";


		private static readonly Manager						m_oManager = Manager.Instance;

		private int											m_nBuildingSpacing;

		private Dictionary<string, ScenarioCity>			m_oCities;
		private Dictionary<string, City>					m_oNames;
		private Dictionary<string, ScenarioCityType>		m_oCityTypes;
		private Dictionary<string, Model>					m_oModels;
		private Dictionary<string, ScenarioTrain>			m_oTrains;
		private Dictionary<string, ScenarioTrainGoodCar>	m_oTrainGoodCars;
		private Dictionary<string, ScenarioTrainTenderCar>	m_oTrainTenderCars;
		private Dictionary<string, ScenarioGood>			m_oGoods;
		private Dictionary<string, ScenarioIndustry>		m_oIndustries;
		private Dictionary<string, ScenarioDepot>			m_oDepots;
		
		private Scenario									m_oBaseScenario;
		private ScenarioSettings							m_oScenarioSettings;
		private ScenarioFiles								m_oFiles;
		private ScenarioState								m_eState = ScenarioState.Unloaded;
		private ScenarioType								m_oType;
		private string										m_sTag;
		private bool										m_bIsDirty;
		private bool										m_bCreateLocalFiles;
		private bool										m_bHasError;

		public StringCollection								Problems = new StringCollection();



		internal Scenario()
		{
		}


		public int BuildingSpacing { get { return m_nBuildingSpacing; } set { m_nBuildingSpacing = value; } }

		public Dictionary<string, ScenarioCity> Cities { get { return m_oCities; } set { m_oCities = value; } }
		public Dictionary<string, City> Names { get { return m_oNames; } set { m_oNames = value; } }
		public Dictionary<string, ScenarioCityType> CityTypes { get { return m_oCityTypes; } set { m_oCityTypes = value; } }
		public Dictionary<string, Model> Models { get { return m_oModels; } set { m_oModels = value; } }
		public Dictionary<string, ScenarioTrain> Trains { get { return m_oTrains; } set { m_oTrains = value; } }
		public Dictionary<string, ScenarioTrainGoodCar> TrainGoodCars { get { return m_oTrainGoodCars; } set { m_oTrainGoodCars = value; } }
		public Dictionary<string, ScenarioTrainTenderCar> TrainTenderCars { get { return m_oTrainTenderCars; } set { m_oTrainTenderCars = value; } }
		public Dictionary<string, ScenarioGood> Goods { get { return m_oGoods; } set { m_oGoods = value; } }
		public Dictionary<string, ScenarioIndustry> Industries { get { return m_oIndustries; } set { m_oIndustries = value; } }
		public Dictionary<string, ScenarioDepot> Depots { get { return m_oDepots; } set { m_oDepots = value; } }


		public Scenario BaseScenario { get { return m_oBaseScenario; } set { m_oBaseScenario = value; } }

		public ScenarioSettings ScenarioSettings { get { return m_oScenarioSettings; } }

		public ScenarioFiles Files { get { return m_oFiles; } set { m_oFiles = value; } }

		public ScenarioState State { get { return m_eState; } }

		public ScenarioType Type { get { return m_oType; } set { m_oType = value; } }

		public string Tag { get { return m_sTag; } set { m_sTag = value; } }

		public bool IsDirty { get { return m_bIsDirty; } set { m_bIsDirty = value; } }

		public bool CreateLocalFiles { get { return m_bCreateLocalFiles; } set { m_bCreateLocalFiles = value; } }

		public bool HasError { get { return m_bHasError; } set { m_bHasError = value; } }



		public static Scenario QuickLoad(string scenarioPath)
		{
			Scenario oScenario = new Scenario();
			oScenario.Tag = Path.GetFileName(scenarioPath);

			oScenario.Files = new ScenarioFiles(scenarioPath, "", "", "", "", "", "", "", "", "", "", "", "", "", "", "");
			oScenario.Load(oScenario.Files, true);
			oScenario.m_eState = ScenarioState.QuickLoaded;

			return oScenario;
		}


		private static void UpdateRootSettings(Scenario scenario)
		{
			foreach (KeyValuePair<string, ScenarioCity> oCityPair in scenario.m_oCities)
			{
				oCityPair.Value.SetRootSetting(scenario.ScenarioSettings);
			}

			foreach (KeyValuePair<string, ScenarioCityType> oCityTypePair in scenario.m_oCityTypes)
			{
				oCityTypePair.Value.SetRootSetting(scenario.ScenarioSettings);
			}

			foreach (KeyValuePair<string, ScenarioIndustry> oIndustryPair in scenario.m_oIndustries)
			{
				oIndustryPair.Value.SetRootSetting(scenario.ScenarioSettings);
			}
		}



		public object Clone()
		{
			if (m_eState != ScenarioState.Loaded)
			{
				Load();
			}

			Scenario oClone = new Scenario();

			if (m_oBaseScenario == null || m_oBaseScenario == this)
			{
				oClone.m_oBaseScenario = this;
			}
			else
			{
				oClone.m_oBaseScenario = this.m_oBaseScenario;
			}

			if (m_oCities != null)
			{
				oClone.m_oCities = new Dictionary<string, ScenarioCity>();
				foreach (KeyValuePair<string, ScenarioCity> oPair in m_oCities)
				{
					oClone.m_oCities.Add(oPair.Key, (ScenarioCity)oPair.Value.Clone());
				}
			}

			if (m_oNames != null)
			{
				oClone.m_oNames = new Dictionary<string, City>();
				foreach (KeyValuePair<string, City> oPair in m_oNames)
				{
					oClone.m_oNames.Add(oPair.Key, (City)oPair.Value.Clone());
				}
			}

			if (m_oCityTypes != null)
			{
				oClone.m_oCityTypes = new Dictionary<string, ScenarioCityType>();
				foreach (KeyValuePair<string, ScenarioCityType> oPair in m_oCityTypes)
				{
					oClone.m_oCityTypes.Add(oPair.Key, (ScenarioCityType)oPair.Value.Clone());
				}
			}

			if (m_oModels != null)
			{
				oClone.m_oModels = new Dictionary<string, Model>();
				foreach (KeyValuePair<string, Model> oPair in m_oModels)
				{
					oClone.m_oModels.Add(oPair.Key, (Model)oPair.Value.Clone());
				}
			}

			if (m_oGoods != null)
			{
				oClone.m_oGoods = new Dictionary<string, ScenarioGood>();
				foreach (KeyValuePair<string, ScenarioGood> oPair in m_oGoods)
				{
					oClone.m_oGoods.Add(oPair.Key, (ScenarioGood)oPair.Value.Clone());
				}
			}

			if (m_oIndustries != null)
			{
				oClone.m_oIndustries = new Dictionary<string, ScenarioIndustry>();
				foreach (KeyValuePair<string, ScenarioIndustry> oPair in m_oIndustries)
				{
					oClone.m_oIndustries.Add(oPair.Key, (ScenarioIndustry)oPair.Value.Clone());
				}
			}

			if (m_oDepots != null)
			{
				oClone.m_oDepots = new Dictionary<string, ScenarioDepot>();
				foreach (KeyValuePair<string, ScenarioDepot> oPair in m_oDepots)
				{
					oClone.m_oDepots.Add(oPair.Key, (ScenarioDepot)oPair.Value.Clone());
				}
			}

			if (m_oTrains != null)
			{
				oClone.m_oTrains = new Dictionary<string, ScenarioTrain>();
				foreach (KeyValuePair<string, ScenarioTrain> oPair in m_oTrains)
				{
					oClone.m_oTrains.Add(oPair.Key, oPair.Value);
				}
			}

			if (m_oTrainGoodCars != null)
			{
				oClone.m_oTrainGoodCars = new Dictionary<string, ScenarioTrainGoodCar>();
				foreach (KeyValuePair<string, ScenarioTrainGoodCar> oPair in m_oTrainGoodCars)
				{
					oClone.m_oTrainGoodCars.Add(oPair.Key, (ScenarioTrainGoodCar)oPair.Value.Clone());
				}
			}

			if (m_oTrainTenderCars != null)
			{
				oClone.m_oTrainTenderCars = new Dictionary<string, ScenarioTrainTenderCar>();
				foreach (KeyValuePair<string, ScenarioTrainTenderCar> oPair in m_oTrainTenderCars)
				{
					oClone.m_oTrainTenderCars.Add(oPair.Key, (ScenarioTrainTenderCar)oPair.Value.Clone());
				}
			}

			oClone.m_nBuildingSpacing = m_nBuildingSpacing;

			oClone.m_oScenarioSettings = (ScenarioSettings)m_oScenarioSettings.Clone();
			oClone.ScenarioSettings.Scenario = oClone;
			oClone.Tag = Guid.NewGuid().ToString();
			oClone.Type = ScenarioType.Local;

			UpdateRootSettings(oClone);

			oClone.m_eState = ScenarioState.Loaded;

			return oClone;
		}


		public override string ToString()
		{
			if (m_oScenarioSettings != null)
			{
				return m_oScenarioSettings.ScenarioName;
			}
			else
			{
				return base.ToString();
			}
		}


		public void Load()
		{
			try
			{
				Load(m_oFiles, false);

				m_eState = ScenarioState.Loaded;
			}
			catch (Exception ex)
			{
				if (!m_oManager.ScenarioErrors.ContainsKey(this.Tag))
				{
					m_oManager.ScenarioErrors[m_sTag] = ex;
				}
				else
				{
					m_oManager.ScenarioErrors.Add(m_sTag, ex);
				}

				m_bHasError = true;
			}
		}


		private void Load(ScenarioFiles files, bool quickLoad)
		{
			if (!quickLoad)
			{
				if (m_eState == ScenarioState.QuickLoaded)
				{
					//Locate scenario files
					m_oFiles = m_oManager.FindScenarioFiles(Path.GetDirectoryName(files.Scenario), this);

					if (m_oType == ScenarioType.Local)
					{
						ScenarioVersionUpdater.UpdateLocalScenario(ManagerApplication.AppSettings, this);
					}

					files = m_oFiles;
				}

				EntityLoader.LoadEntities(this, files);
			}

			LoadScenario(files.Scenario, quickLoad);

			if (!quickLoad)
			{
				UpdateRootSettings(this);
			}
		}


		private void LoadScenario(string path, bool quickLoad)
		{
			Problems = new StringCollection();

			XmlDocument oDoc = new XmlDocument();
			using (StreamReader oReader = new StreamReader(path, Encoding.UTF8))
			{
				oDoc.Load(oReader);
				oReader.Close();
			}

			XmlNode oNode = oDoc.SelectSingleNode("/RRTScenario");

			if (m_oScenarioSettings == null)
			{
				m_oScenarioSettings = new ScenarioSettings(this);
			}

			m_oScenarioSettings.FromXml(oNode, !quickLoad);
		}


		public void Save()
		{
			PrepareSupportFiles();

			SaveSupportFiles();

			SaveScenario(m_oFiles.Scenario);

			m_eState = ScenarioState.Loaded;
			m_bIsDirty = false;
		}


		private void PrepareSupportFiles()
		{
			foreach (KeyValuePair<string, ScenarioGood> oPair in m_oGoods)
			{
				//Make sure required depots are added to this scenario
				string[] aDepotNames = DepotGoodMapper.GetDepot(oPair.Value);
				if ((aDepotNames != null) && (aDepotNames.Length > 0))
				{
					foreach (string sDepotName in aDepotNames)
					{
						if (!m_oDepots.ContainsKey(sDepotName))
						{
							ScenarioDepot oDepot = ScenarioDepot.FromDepot(m_oManager.BaseDepots[sDepotName]);
							m_oDepots.Add(oDepot.Name, oDepot);
						}
					}
				}

				//Make sure required TrainGoodCars are added to this scenario
				foreach (TrainGoodCar oCar in oPair.Value.TrainCars)
				{
					ScenarioTrainGoodCar oScenarioGoodCar = oCar as ScenarioTrainGoodCar;
					if (oScenarioGoodCar != null)
					{
						if (!m_oTrainGoodCars.ContainsKey(oCar.Name))
						{
							m_oTrainGoodCars.Add(oScenarioGoodCar.Name, oScenarioGoodCar);
						}
					}
				}

				/*
				//Make sure required TrainTenderCars are added to this scenario
				foreach (TrainTenderCar oCar in oPair.Value.TrainCars)
				{
					ScenarioTrainTenderCar oScenarioTenderCar = oCar as ScenarioTrainTenderCar;
					if (oScenarioTenderCar != null)
					{
						if (!m_oTrainTenderCars.ContainsKey(oCar.Name))
						{
							m_oTrainTenderCars.Add(oScenarioTenderCar.Name, oScenarioTenderCar);
						}
					}
				}
				*/
			}
		}


		private void SaveSupportFiles()
		{
			if (m_oNames != null)
			{
				SaveSupportFile(m_oFiles.Names, m_oNames, "Names", "RRTCities", "TextKey", "Tag");
			}

			if (m_oGoods != null)
			{
				SaveSupportFile(m_oFiles.Goods, m_oGoods, "RRTGoods", "", "Good", "szName");
			}

			SaveSupportValue(m_oFiles.Cities, m_nBuildingSpacing, "RRTCities", "GridSize", "BuildingSpacing");

			if (m_oCityTypes != null)
			{
				SaveSupportFile(m_oFiles.Cities, m_oCityTypes, "RRTCities", "RRTCityTypes", "CityType", "szName");
			}

			if (m_oModels != null)
			{
				SaveSupportFile(m_oFiles.Cities, m_oModels, "RRTCities", "RRTModels", "Model", "szName");
			}

			if (m_oCities != null)
			{
				SaveSupportFile(m_oFiles.Cities, m_oCities, "RRTCities", "", "City", "szName");
			}

			if (m_oIndustries != null)
			{
				SaveSupportFile(m_oFiles.Industries, m_oIndustries, "RRTIndustries", "Industries", "RRTIndustry", "szName");
			}

			if (m_oDepots != null)
			{
				SaveSupportFile(m_oFiles.Depots, m_oDepots, "RRTDepots", "", "Depot", "szName");
			}

			if (m_oTrains != null)
			{
				SaveSupportFile(m_oFiles.Trains, m_oTrains, "RRTTrains", "", "Train", "szName");
			}

			if (m_oTrainGoodCars != null)
			{
				SaveSupportFile(m_oFiles.TrainCars, m_oTrainGoodCars, "RRTTrainCars", "RRTGoodCars", "TrainCar", "szName");
			}

			if (m_oTrainTenderCars != null)
			{
				SaveSupportFile(m_oFiles.TrainCars, m_oTrainTenderCars, "RRTTrainCars", "RRTTenderCars", "TenderCar", "szName");
			}
		}


		private void SaveSupportFile(string path, IDictionary entities, string rootName, string subRoot, string elementName, string key)
		{
			XmlDocument oDoc = new XmlDocument();
			XmlNode oRoot;

			if (File.Exists(path))
			{
				using (StreamReader oReader = new StreamReader(path, Encoding.UTF8))
				{
					oDoc.Load(oReader);
					oReader.Close();
				}

				//Remove any entities not in the list
				XmlNodeList oList;
				if (string.IsNullOrEmpty(subRoot))
				{
					oList = oDoc.SelectNodes("/" + rootName + "/" + elementName);
				}
				else
				{
					oList = oDoc.SelectNodes("/" + rootName + "/" + subRoot + "/" + elementName);
				}

				if ((oList != null) && (oList.Count > 0))
				{
					foreach (XmlNode oNode in oList)
					{
						XmlNode oKeyNode = oNode.SelectSingleNode(key);
						if (oKeyNode != null)
						{
							if (!entities.Contains(oKeyNode.InnerText))
							{
								oNode.ParentNode.RemoveChild(oNode);
							}
						}
					}
				}

				oRoot = oDoc.SelectSingleNode("/" + rootName);
				if (oRoot == null)
				{
					oRoot = Utility.CreateChildElement(oDoc, rootName);
				}

				if (!string.IsNullOrEmpty(subRoot))
				{
					XmlNode oChild = oRoot.SelectSingleNode(subRoot);
					if (oChild == null)
					{
						oChild = Utility.CreateChildElement(oRoot, subRoot);
					}

					oRoot = oChild;
				}
			}
			else
			{
				if (!Directory.Exists(Path.GetDirectoryName(path)))
				{
					Directory.CreateDirectory(Path.GetDirectoryName(path));
				}

				XmlDeclaration oXmlDeclaration = oDoc.CreateXmlDeclaration("1.0", "utf-8", null);
				oDoc.InsertBefore(oXmlDeclaration, oDoc.DocumentElement);

				XmlElement oElement = oDoc.CreateElement(rootName);
				oDoc.AppendChild(oElement);

				if (!string.IsNullOrEmpty(subRoot))
				{
					XmlElement oSubElement = oDoc.CreateElement(subRoot);
					oElement.AppendChild(oSubElement);

					oRoot = oSubElement;
				}
				else
				{
					oRoot = oElement;
				}
			}

			foreach (DictionaryEntry oEntry in entities)
			{
				((XmlSetting)oEntry.Value).ToXml(oRoot);
			}

			Manager.WriteXmlDocument(oDoc, path, Encoding.ASCII);
		}


		private void SaveSupportValue(string path, object value, string rootName, string subRoot, string elementName)
		{
			XmlDocument oDoc = new XmlDocument();
			XmlNode oRoot;

			if (File.Exists(path))
			{
				using (StreamReader oReader = new StreamReader(path, Encoding.UTF8))
				{
					oDoc.Load(oReader);
					oReader.Close();
				}

				oRoot = oDoc.SelectSingleNode("/" + rootName);
				if (oRoot == null)
				{
					oRoot = Utility.CreateChildElement(oDoc, rootName);
				}

				if (!string.IsNullOrEmpty(subRoot))
				{
					XmlNode oChild = oRoot.SelectSingleNode(subRoot);
					if (oChild == null)
					{
						oChild = Utility.CreateChildElement(oRoot, subRoot);
					}

					oRoot = oChild;
				}
			}
			else
			{
				if (!Directory.Exists(Path.GetDirectoryName(path)))
				{
					Directory.CreateDirectory(Path.GetDirectoryName(path));
				}

				XmlDeclaration oXmlDeclaration = oDoc.CreateXmlDeclaration("1.0", "utf-8", null);
				oDoc.InsertBefore(oXmlDeclaration, oDoc.DocumentElement);

				XmlElement oElement = oDoc.CreateElement(rootName);
				oDoc.AppendChild(oElement);

				if (!string.IsNullOrEmpty(subRoot))
				{
					XmlElement oSubElement = oDoc.CreateElement(subRoot);
					oElement.AppendChild(oSubElement);

					oRoot = oSubElement;
				}
				else
				{
					oRoot = oElement;
				}
			}

			Utility.SetChildElementValue(oRoot, elementName, value);

			Manager.WriteXmlDocument(oDoc, path, Encoding.ASCII);
		}


		private void SaveScenario(string path)
		{
			//All saves happen in the local data directory
			XmlDocument oDoc = new XmlDocument();

			//Load existing local scenario if exists
			if (File.Exists(path))
			{
				using (StreamReader oReader = new StreamReader(path, Encoding.UTF8))
				{
					oDoc.Load(oReader);
					oReader.Close();
				}
			}
			else
			{
				if (!Directory.Exists(Path.GetDirectoryName(path)))
				{
					Directory.CreateDirectory(Path.GetDirectoryName(path));
				}

				XmlDeclaration oXmlDeclaration = oDoc.CreateXmlDeclaration("1.0", "utf-8", null);
				oDoc.InsertBefore(oXmlDeclaration, oDoc.DocumentElement);

				XmlElement oElement = oDoc.CreateElement("RRTScenario");
				oDoc.AppendChild(oElement);
			}

			m_oScenarioSettings.ToXml(oDoc.DocumentElement);

			Manager.WriteXmlDocument(oDoc, path, Encoding.ASCII);
		}


		public void Delete()
		{
			if ((m_oBaseScenario == null) || (m_oBaseScenario == this))
			{
				m_oManager.UserScenarios.Remove(m_sTag);
			}
			else
			{
				m_oManager.LocalScenarios.Remove(m_sTag);
			}

			Utility.DeleteDirectory(Path.GetDirectoryName(m_oFiles.Scenario));
		}


		public bool Validate(List<string> errors)
		{
			if (errors == null)
			{
				errors = new List<string>();
			}

			return m_oScenarioSettings.Validate(errors);
		}


		internal void SetProperty(ref string property, string value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref int property, int value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}
		
		internal void SetProperty(ref long property, long value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref bool property, bool value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref float property, float value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref double property, double value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref TrackSideType property, TrackSideType value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref Color property, Color value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref IndustrySize property, IndustrySize value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref ScenarioCity property, ScenarioCity value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref SpeedRecordType property, SpeedRecordType value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref ScenarioTrain property, ScenarioTrain value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref ScenarioGood property, ScenarioGood value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref Good property, Good value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref Industry property, Industry value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref OwnerType property, OwnerType value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

		internal void SetProperty(ref ScenarioCityType property, ScenarioCityType value)
		{
			if (property != value) { property = value; m_bIsDirty = true; }
		}

	}
}