using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class ScenarioCityType : XmlSetting
	{
		public static ScenarioCityType AnyCityType = new ScenarioCityType("<Any Type>");

		private List<Resource>	m_oProductionModifiers = new List<Resource>();
		private string			m_sName;
		private int				m_nSize;
		private int				m_nLargeBuildings;
		private int				m_nMediumBuildings;
		private int				m_nSmallBuildings;


		public ScenarioCityType()
		{
		}

		internal ScenarioCityType(string name)
		{
			m_sName = name;
		}

		public ScenarioCityType(ScenarioSettings rootSetting)
		{
			base.RootSetting = rootSetting;
		}


		public class Resource : Industry.Resource
		{
			private float m_fMaxStorage;


			public Resource()
			{
			}

			public Resource(ScenarioSettings rootSetting)
			{
				base.RootSetting = rootSetting;
			}


			public float MaxStorage { get { return m_fMaxStorage; } set { Scenario.SetProperty(ref m_fMaxStorage, value); } }


			public override void ToXml(XmlNode node)
			{
				XmlNode oResourceNode = base.ToXml_Create(node);
				if (oResourceNode != null)
				{
					Utility.SetChildElementValue(oResourceNode, "MaxStorage", m_fMaxStorage);
				}
			}


			public override void FromXml(Scenario scenario, XmlNode node)
			{
				base.FromXml(scenario, node);
				m_fMaxStorage = Utility.GetChildElementValue<float>(node, "MaxStorage");
			}


			public override object Clone()
			{
				Resource oClone = new Resource();
				oClone.m_oInput = (Good)base.Input.Clone();
				oClone.m_oOutput = (Good)base.Output.Clone();
				oClone.m_oInputOutputRatio = base.InputOutputRatio;
				oClone.m_fMaxStorage = m_fMaxStorage;
				return oClone;
			}


			public override bool Validate(List<string> errors)
			{
				return true;
			}
		}


		public List<Resource> ProductionModifiers { get { return m_oProductionModifiers; } set { m_oProductionModifiers = value; } }

		protected Scenario Scenario { get { return (base.RootSetting as ScenarioSettings).Scenario; } }

		public string Name { get { return m_sName; } set { Scenario.SetProperty(ref m_sName, value); } }
		public int Size { get { return m_nSize; } set { Scenario.SetProperty(ref m_nSize, value); } }
		public int LargeBuildings { get { return m_nLargeBuildings; } set { Scenario.SetProperty(ref m_nLargeBuildings, value); } }
		public int MediumBuildings { get { return m_nMediumBuildings; } set { Scenario.SetProperty(ref m_nMediumBuildings, value); } }
		public int SmallBuildings { get { return m_nSmallBuildings; } set { Scenario.SetProperty(ref m_nSmallBuildings, value); } }


		public override void ToXml(XmlNode node)
		{
			XmlNode oCityNode = Utility.GetOrCreateChildElement(node, "CityType[szName='" + m_sName + "']", "CityType");

			Utility.SetChildElementValue(oCityNode, "szName", m_sName);
			Utility.SetChildElementValue(oCityNode, "Size", m_nSize);
			Utility.SetChildElementValue(oCityNode, "NumLargeBuildings", m_nLargeBuildings);
			Utility.SetChildElementValue(oCityNode, "NumMediumBuildings", m_nMediumBuildings);
			Utility.SetChildElementValue(oCityNode, "NumSmallBuildings", m_nSmallBuildings);

			Utility.RemoveElements(oCityNode, "ProductionModifiers");

			if ((m_oProductionModifiers != null) && (m_oProductionModifiers.Count > 0))
			{
				XmlNode oModifiersNode = Utility.GetOrCreateChildElement(oCityNode, "ProductionModifiers", "ProductionModifiers");
				foreach (Resource oResource in m_oProductionModifiers)
				{
					oResource.ToXml(oModifiersNode);
				}
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			m_sName = Utility.GetChildElementValue<string>(node, "szName");
			m_nSize = Utility.GetChildElementValue<int>(node, "Size", 0, false);
			m_nLargeBuildings = Utility.GetChildElementValue<int>(node, "NumLargeBuildings", 0, false);
			m_nMediumBuildings = Utility.GetChildElementValue<int>(node, "NumMediumBuildings", 0, false);
			m_nSmallBuildings = Utility.GetChildElementValue<int>(node, "NumSmallBuildings", 0, false);

			m_oProductionModifiers = base.LoadSubSetting<Resource>(node, "ProductionModifiers/Resource");
		}


		public override object Clone()
		{
			ScenarioCityType oClone = new ScenarioCityType();

			oClone.m_nSize = m_nSize;
			oClone.m_nLargeBuildings = m_nLargeBuildings;
			oClone.m_nMediumBuildings = m_nMediumBuildings;
			oClone.m_sName = m_sName;
			oClone.m_nSmallBuildings = m_nSmallBuildings;

			oClone.m_oProductionModifiers = new List<Resource>();
			foreach (Resource resource in m_oProductionModifiers)
			{
				oClone.m_oProductionModifiers.Add((Resource)resource.Clone());
			}

			return oClone;
		}


		public override bool Validate(List<string> errors)
		{
			return true;
		}


		internal void SetRootSetting(XmlSetting root)
		{
			RootSetting = root;

			foreach (Resource oResource in m_oProductionModifiers)
			{
				oResource.RootSetting = root;
			}
		}

		public override string ToString()
		{
			return m_sName;
		}
	}
}