using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class ScenarioGood : Good
	{
		public static ScenarioGood AnyGood = new ScenarioGood("<Any Good>");
		public static ScenarioGood NoGood = new ScenarioGood("<None>");


		private int m_nMinPrice = -1;
		private int m_nMaxPrice = -1;


		public ScenarioGood()
		{
		}


		public ScenarioGood(string name)
		{
			base.Name = name;
		}


		public static ScenarioGood FromGood(Good good)
		{
			ScenarioGood oNewGood = new ScenarioGood();

			oNewGood.Name = good.Name;

			oNewGood.TrainCars = new List<TrainGoodCar>();
			foreach (TrainGoodCar oCar in good.TrainCars)
			{
				oNewGood.TrainCars.Add(ScenarioTrainGoodCar.FromTrainGoodCar(oCar));
			}

			return oNewGood;
		}


		public new int MinPrice { get { return m_nMinPrice; } set { m_nMinPrice = value; } }

		public new int MaxPrice { get { return m_nMaxPrice; } set { m_nMaxPrice = value; } }


		public override void ToXml(XmlNode node)
		{
			XmlNode oGood = Utility.GetOrCreateChildElement(node, "Good[szName='" + base.Name + "']", "Good");

			if (this == AnyGood)
			{
				Utility.SetChildElementValue(oGood, "szName", Scenario.ANY_GOOD);
			}
			else
			{
				Utility.SetChildElementValue(oGood, "szName", base.Name);

				if (m_nMinPrice != -1)
				{
					Utility.SetChildElementValue(oGood, "MinPrice", m_nMinPrice);
				}
				else
				{
					Utility.RemoveElements(oGood, "MinPrice");
				}

				if (m_nMaxPrice != -1)
				{
					Utility.SetChildElementValue(oGood, "MaxPrice", m_nMaxPrice);
				}
				else 
				{
					Utility.RemoveElements(oGood, "MaxPrice");
				}
			}
		}


		public override void FromXml(Scenario scenario, XmlNode node)
		{
			base.Name = Utility.GetChildElementValue<string>(node, "szName");
			m_nMinPrice = Utility.GetChildElementValue<int>(node, "MinPrice", -1, false);
			m_nMaxPrice = Utility.GetChildElementValue<int>(node, "MaxPrice", -1, false);
			
			Good oBaseGood;
			base.TrainCars = new List<TrainGoodCar>();
			if (Manager.Instance.BaseGoods.TryGetValue(base.Name, out oBaseGood))
			{
				foreach (TrainGoodCar oCar in oBaseGood.TrainCars)
				{
					base.TrainCars.Add(ScenarioTrainGoodCar.FromTrainGoodCar(oCar));
				}
			}
		}


		public override object Clone()
		{
			ScenarioGood oClone = new ScenarioGood();

			oClone.Name = base.Name;
			oClone.m_nMinPrice = m_nMinPrice;
			oClone.m_nMaxPrice = m_nMaxPrice;

			oClone.TrainCars = new List<TrainGoodCar>();
			foreach (ScenarioTrainGoodCar oCar in this.TrainCars)
			{
				oClone.TrainCars.Add((ScenarioTrainGoodCar)oCar.Clone());
			}

			return oClone;
		}


		public override bool Validate(List<string> errors)
		{
			return true;
		}
	}
}
