using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Xml;

namespace Railroads.ScenarioManager.Core
{
	public class ScenarioSettings : XmlSetting
	{
		private Scenario		m_oScenario;

		private List<Era>		m_oEras = new List<Era>();
		private Version?		m_oManagerVersion;
		private string			m_sSkyBoxName;
		private string			m_sWallFiles;
		private string			m_sCurrency;
		private string			m_sAllAboardSoundID;
		private Color			m_crShallowColor;
		private Color			m_crDeepColor;
		private string			m_sScenarioName;
		private string			m_sSubtitle;
		private string			m_sAuthor;
		private string			m_sDescription;
		private int				m_nMapSize;
		private string			m_sMapFile;
		private string			m_sMapThumbnail;
		private bool			m_bIsMultiplayer;
		private string			m_sCitiesFile;
		private string			m_sNamesFile;
		private string			m_sIndustriesFile;
		private string			m_sDepotsFile;
		private string			m_sGoodsFile;
		private string			m_sEventsFile;
		private string			m_sTrainsFile;
		private string			m_sTrainCarsFile;
		private string			m_sErasFile;
		private string			m_sDecorationsFile;
		private string			m_sTreesFile;
		private string			m_sClutterFile;
		private string			m_sBridgesFile;
		private string			m_sTunnelsFile;
		private string			m_sQuayleFile;
		private int				m_nTurnsPerMonth;

		private List<string>	m_oAICharacters = new List<string>();

		
		internal ScenarioSettings(Scenario scenario)
		{
			m_oScenario = scenario;

			this.RootSetting = this;
		}


		public Scenario Scenario { get { return m_oScenario; } set { m_oScenario = value; } }

		public List<Era> Eras { get { return m_oEras; } }

		public Version? ManagerVersion { get { return m_oManagerVersion; } set { m_oManagerVersion = value; } }

		public Color ShallowColor { get { return m_crShallowColor; } set { Scenario.SetProperty(ref m_crShallowColor, value); } }
		public Color DeepColor { get { return m_crDeepColor; } set { Scenario.SetProperty(ref m_crDeepColor, value); } }
		public string ScenarioName { get { return m_sScenarioName; } set { Scenario.SetProperty(ref m_sScenarioName, value); } }
		public string Subtitle { get { return m_sSubtitle; } set { Scenario.SetProperty(ref m_sSubtitle, value); } }
		public string Author { get { return m_sAuthor; } set { Scenario.SetProperty(ref m_sAuthor, value); } }
		public string Description { get { return m_sDescription; } set { Scenario.SetProperty(ref m_sDescription, value); } }
		public int TurnsPerMonth { get { return m_nTurnsPerMonth; } set { Scenario.SetProperty(ref m_nTurnsPerMonth, value); } }
		public int MapSize { get { return m_nMapSize; } set { Scenario.SetProperty(ref m_nMapSize, value); } }
		public string MapFile { get { return m_sMapFile; } set { Scenario.SetProperty(ref m_sMapFile, value); } }
		public string MapThumbnail { get { return m_sMapThumbnail; } set { Scenario.SetProperty(ref m_sMapThumbnail, value); } }
		public bool IsMultiplayer { get { return m_bIsMultiplayer; } set { Scenario.SetProperty(ref m_bIsMultiplayer, value); } }

		public string SkyBoxName { get { return m_sSkyBoxName; } set { Scenario.SetProperty(ref m_sSkyBoxName, value); } }
		public string WallFiles { get { return m_sWallFiles; } set { Scenario.SetProperty(ref m_sWallFiles, value); } }
		public string Currency { get { return m_sCurrency; } set { Scenario.SetProperty(ref m_sCurrency, value); } }
		public string AllAboardSoundID { get { return m_sAllAboardSoundID; } set { Scenario.SetProperty(ref m_sAllAboardSoundID, value); } }


		public string CitiesFile { get { return m_sCitiesFile; } set { m_sCitiesFile = value; } }
		public string NamesFile { get { return m_sNamesFile; } set { m_sNamesFile = value; } }
		public string IndustriesFile { get { return m_sIndustriesFile; } set { m_sIndustriesFile = value; } }
		public string DepotsFile { get { return m_sDepotsFile; } set { m_sDepotsFile = value; } }
		public string GoodsFile { get { return m_sGoodsFile; } set { m_sGoodsFile = value; } }
		public string TrainsFile { get { return m_sTrainsFile; } set { m_sTrainsFile = value; } }
		public string TrainCarsFile { get { return m_sTrainCarsFile; } set { m_sTrainCarsFile = value; } }
		public string EventsFile { get { return m_sEventsFile; } set { m_sEventsFile = value; } }
		public string ErasFile { get { return m_sErasFile; } set { m_sErasFile = value; } }
		public string DecorationsFile { get { return m_sDecorationsFile; } set { m_sDecorationsFile = value; } }
		public string TreesFile { get { return m_sTreesFile; } set { m_sTreesFile = value; } }
		public string ClutterFile { get { return m_sClutterFile; } set { m_sClutterFile = value; } }
		public string BridgesFile { get { return m_sBridgesFile; } set { m_sBridgesFile = value; } }
		public string TunnelsFile { get { return m_sTunnelsFile; } set { m_sTunnelsFile = value; } }
		public string QuayleFile { get { return m_sQuayleFile; } set { m_sQuayleFile = value; } }


		public override void ToXml(XmlNode node)
		{
			//Added check to only persist base scenario if it has one and it is not itself
			if ((Scenario.BaseScenario != null) && (Scenario.BaseScenario != Scenario))
			{
				//Utility.SetChildElementValue(node, "szBaseScenario", Scenario.BaseScenario.Tag);
			}

			Utility.SetChildElementValue(node, "szMapName", m_sScenarioName);
			Utility.SetChildElementValue(node, "szSubtitle", m_sSubtitle);
			Utility.SetChildElementValue(node, "szDescription", m_sDescription);
			Utility.SetChildElementValue(node, "szAuthor", m_sAuthor);
			Utility.SetChildElementValue(node, "MapSize", m_nMapSize);
			if (m_nTurnsPerMonth != 16) Utility.SetChildElementValue(node, "iTurnsPerMonth", m_nTurnsPerMonth);

			if (m_bIsMultiplayer) Utility.SetChildElementValue(node, "bIsMultiplayer", m_bIsMultiplayer);
			//Utility.SetChildElementValue(node, "managerVersion", ManagerApplication.ManagerVersion);


			Utility.SetChildElementValue(node, "BridgeXMLFile", m_sBridgesFile);
			Utility.SetChildElementValue(node, "CityXMLFile", m_sCitiesFile);
			Utility.SetChildElementValue(node, "ClutterXMLFile", m_sClutterFile);
			Utility.SetChildElementValue(node, "DecorationsXMLFile", m_sDecorationsFile);
			Utility.SetChildElementValue(node, "DepotsXMLFile", m_sDepotsFile);
			Utility.SetChildElementValue(node, "EraXMLFile", m_sErasFile);
			Utility.SetChildElementValue(node, "EventsXMLFile", m_sEventsFile);
			Utility.SetChildElementValue(node, "GoodsXMLFile", m_sGoodsFile);
			Utility.SetChildElementValue(node, "IndustriesXMLFile", m_sIndustriesFile);
			Utility.SetChildElementValue(node, "NamesXMLFile", m_sNamesFile);
			Utility.SetChildElementValue(node, "QuayleXMLFile", m_sQuayleFile);
			Utility.SetChildElementValue(node, "TrainCarsXMLFile", m_sTrainCarsFile);
			Utility.SetChildElementValue(node, "TrainsXMLFile", m_sTrainsFile);
			Utility.SetChildElementValue(node, "TreesXMLFile", m_sTreesFile);
			Utility.SetChildElementValue(node, "TunnelXMLFile", m_sTunnelsFile);

			Utility.SetChildElementValue(node, "szSkyBoxName", m_sSkyBoxName);

			Utility.SetChildElementValue(node, "szShallowWaterColor", Utility.ColorToRgbString(m_crShallowColor));
			Utility.SetChildElementValue(node, "szDeepWaterColor", Utility.ColorToRgbString(m_crDeepColor));

			Utility.SetChildElementValue(node, "WallFiles", m_sWallFiles);
			Utility.SetChildElementValue(node, "Currency", m_sCurrency);
			Utility.SetChildElementValue(node, "szAllAboardSoundID", m_sAllAboardSoundID);

			Utility.SetChildElementValue(node, "szMapFileName", m_sMapFile);
			Utility.SetChildElementValue(node, "szMapThumbnail", m_sMapThumbnail);


			Utility.RemoveElements(node, "Eras");

			XmlNode oEras = Utility.CreateChildElement(node, "Eras");
			foreach (Era oEra in m_oEras)
			{
				oEra.ToXml(oEras);
			}

			Utility.RemoveElements(node, "AICharacters");

			XmlNode oAICharacters = Utility.CreateChildElement(node, "AICharacters");
			foreach (string s in m_oAICharacters)
			{
				XmlNode oChild = node.OwnerDocument.CreateElement("szName");
				oChild.InnerText = s;
				oAICharacters.AppendChild(oChild);
			}
		}

		public override void FromXml(Scenario scenario, XmlNode node)
		{
			FromXml(node, true);
		}

		public void FromXml(XmlNode node, bool loadAll)
		{
			XmlNode oRoot = node.SelectSingleNode("/RRTScenario");

			m_sScenarioName = Utility.GetChildElementValue<string>(oRoot, "szMapName");
			m_sSubtitle = Utility.GetChildElementValue<string>(oRoot, "szSubtitle", "", false);
			m_sDescription = Utility.GetChildElementValue<string>(oRoot, "szDescription", "", false);
			m_sAuthor = Utility.GetChildElementValue<string>(oRoot, "szAuthor", "", false);
			m_nTurnsPerMonth = Utility.GetChildElementValue<int>(oRoot, "iTurnsPerMonth", 16, false);
			m_nMapSize = Utility.GetChildElementValue<int>(oRoot, "MapSize", 10, false);

			m_sBridgesFile = Utility.GetChildElementValue<string>(oRoot, "BridgeXMLFile", m_sBridgesFile, false);
			m_sCitiesFile = Utility.GetChildElementValue<string>(oRoot, "CityXMLFile");
			m_sClutterFile = Utility.GetChildElementValue<string>(oRoot, "ClutterXMLFile", m_sClutterFile, false);
			m_sDecorationsFile = Utility.GetChildElementValue<string>(oRoot, "DecorationsXMLFile", m_sDecorationsFile, false);
			m_sDepotsFile = Utility.GetChildElementValue<string>(oRoot, "DepotsXMLFile", m_sDepotsFile, false);
			m_sErasFile = Utility.GetChildElementValue<string>(oRoot, "EraXMLFile", m_sErasFile, false);
			m_sEventsFile = Utility.GetChildElementValue<string>(oRoot, "EventsXMLFile", m_sEventsFile, false);
			m_sGoodsFile = Utility.GetChildElementValue<string>(oRoot, "GoodsXMLFile");
			m_sIndustriesFile = Utility.GetChildElementValue<string>(oRoot, "IndustriesXMLFile");
			m_sNamesFile = Utility.GetChildElementValue<string>(oRoot, "NamesXMLFile", m_sNamesFile, false);
			m_sQuayleFile = Utility.GetChildElementValue<string>(oRoot, "QuayleXMLFile", m_sQuayleFile, false);
			m_sTrainCarsFile = Utility.GetChildElementValue<string>(oRoot, "TrainCarsXMLFile", m_sTrainCarsFile, false);
			m_sTrainsFile = Utility.GetChildElementValue<string>(oRoot, "TrainsXMLFile");
			m_sTreesFile = Utility.GetChildElementValue<string>(oRoot, "TreesXMLFile", m_sTreesFile, false);
			m_sTunnelsFile = Utility.GetChildElementValue<string>(oRoot, "TunnelXMLFile", m_sTunnelsFile, false);

			string sTemp = Utility.GetChildElementValue<string>(oRoot, "managerVersion", "", false);
			if (!string.IsNullOrEmpty(sTemp))
			{
				m_oManagerVersion = Version.FromString(sTemp);
			}
			else
			{
				m_oManagerVersion = null;
			}

			//Multiplayer maps have a _MP after the scenario file prefix
			if (Path.GetFileName(m_oScenario.Files.Scenario).Contains(Constants.SCENARIO_FILE + "_MP"))
			{
				m_bIsMultiplayer = true;
			}
			else
			{
				m_bIsMultiplayer = false;
			}

			if (!m_bIsMultiplayer)
			{
				m_bIsMultiplayer = Utility.GetChildElementValue<bool>(oRoot, "bIsMultiplayer", false, false);
			}

			string sBaseScenarioTag = Utility.GetChildElementValue<string>(oRoot, "szBaseScenario", "", false);
			if (!string.IsNullOrEmpty(sBaseScenarioTag))
			{
				Scenario oBaseScenario;
				if (!Manager.Instance.CoreScenarios.TryGetValue(sBaseScenarioTag, out oBaseScenario))
				{
					if (!Manager.Instance.UserScenarios.TryGetValue(sBaseScenarioTag, out oBaseScenario))
					{
						m_oScenario.HasError = true;
						Manager.Instance.ScenarioErrors.Add(m_oScenario.Tag, new ArgumentException("Could not find base scenario '" + sBaseScenarioTag + "'"));
						return;
					}
				}

				if (oBaseScenario != null)
				{
					this.Scenario.BaseScenario = oBaseScenario;
				}
			}

			if (loadAll)
			{
				m_sSkyBoxName = Utility.GetChildElementValue<string>(oRoot, "szSkyBoxName", "", false);

				m_crShallowColor = Utility.ToColor(Utility.GetChildElementValue<string>(oRoot, "szShallowWaterColor"));
				m_crDeepColor = Utility.ToColor(Utility.GetChildElementValue<string>(oRoot, "szDeepWaterColor"));

				m_sWallFiles = Utility.GetChildElementValue<string>(oRoot, "WallFiles", "", false);
				m_sCurrency = Utility.GetChildElementValue<string>(oRoot, "Currency", "", false);
				m_sAllAboardSoundID = Utility.GetChildElementValue<string>(oRoot, "szAllAboardSoundID", "", false);


				m_sMapFile = Utility.GetChildElementValue<string>(oRoot, "szMapFileName", "", false);
				m_sMapThumbnail = Utility.GetChildElementValue<string>(oRoot, "szMapThumbnail", "", false);

				if (string.IsNullOrEmpty(m_sMapFile) && m_oScenario.BaseScenario != null)
				{
					m_sMapFile = m_oScenario.BaseScenario.ScenarioSettings.MapFile;
				}
				if (string.IsNullOrEmpty(m_sMapThumbnail) && m_oScenario.BaseScenario != null)
				{
					m_sMapThumbnail = m_oScenario.BaseScenario.ScenarioSettings.MapThumbnail;
				}

				XmlNodeList oErasList = oRoot.SelectNodes("Eras/Era");
				m_oEras = new List<Era>();
				if ((oErasList != null) && (oErasList.Count > 0))
				{
					foreach (XmlNode oEraNode in oErasList)
					{
						Era oEra = new Era();
						oEra.RootSetting = this;
						oEra.FromXml(m_oScenario, oEraNode);
						m_oEras.Add(oEra);
					}
				}

				XmlNodeList oAICharactersList = oRoot.SelectNodes("AICharacters/szName");
				m_oAICharacters = new List<string>();
				if ((oAICharactersList != null) && (oAICharactersList.Count > 0))
				{
					foreach (XmlNode oNameNode in oAICharactersList)
					{
						m_oAICharacters.Add(oNameNode.InnerText);
					}
				}
			}
		}

		public override object Clone()
		{
			ScenarioSettings oClone = new ScenarioSettings(null);

			oClone.m_sAuthor = m_sAuthor;
			oClone.m_crDeepColor = m_crDeepColor;
			oClone.m_sDescription = m_sDescription;
			oClone.m_sScenarioName = m_sScenarioName;
			oClone.m_crShallowColor = m_crShallowColor;
			oClone.m_sSubtitle = m_sSubtitle;
			oClone.m_nMapSize = m_nMapSize;
			oClone.m_nTurnsPerMonth = m_nTurnsPerMonth;

			oClone.m_sSkyBoxName = m_sSkyBoxName;
			oClone.m_sWallFiles = m_sWallFiles;
			oClone.m_sCurrency = m_sCurrency;
			oClone.m_sAllAboardSoundID = m_sAllAboardSoundID;

			oClone.m_sCitiesFile = m_sCitiesFile;
			oClone.m_sNamesFile = m_sNamesFile;
			oClone.m_sIndustriesFile = m_sIndustriesFile;
			oClone.m_sDepotsFile = m_sDepotsFile;
			oClone.m_sGoodsFile = m_sGoodsFile;
			oClone.m_sTrainsFile = m_sTrainsFile;
			oClone.m_sTrainCarsFile = m_sTrainCarsFile;
			oClone.m_sEventsFile = m_sEventsFile;
			oClone.m_sErasFile = m_sErasFile;
			oClone.m_sDecorationsFile = m_sDecorationsFile;
			oClone.m_sTreesFile = m_sTreesFile;
			oClone.m_sClutterFile = m_sClutterFile;
			oClone.m_sBridgesFile = m_sBridgesFile;
			oClone.m_sTunnelsFile = m_sTunnelsFile;
			oClone.m_sQuayleFile = m_sQuayleFile;

			oClone.m_sMapFile = m_sMapFile;
			oClone.m_sMapThumbnail = m_sMapThumbnail;

			oClone.m_bIsMultiplayer = m_bIsMultiplayer;

			oClone.m_oEras = new List<Era>();
			foreach (Era oEra in m_oEras)
			{
				Era oCloneEra = (Era)oEra.Clone();
				oCloneEra.SetRootSetting(oClone);
				oClone.m_oEras.Add(oCloneEra);
			}

			oClone.m_oAICharacters = new List<string>();
			foreach (string s in m_oAICharacters)
			{
				oClone.m_oAICharacters.Add(s);
			}

			return oClone;
		}


		public override bool Validate(List<string> errors)
		{
			if (string.IsNullOrEmpty(m_sScenarioName))
			{
				errors.Add("A scenario name must be defined.");
				return false;
			}

			if (string.IsNullOrEmpty(m_sAuthor))
			{
				errors.Add("You must define a scenario author.");
				return false;
			}

			if (!CheckMapFile(m_sMapThumbnail))
			{
				errors.Add("Could not find the map thumbnail file.");
				return false;
			}

			if (!CheckMapFile(m_sMapFile))
			{
				errors.Add("Could not find the map file.");
				return false;
			}

			if (m_oEras.Count == 0)
			{
				errors.Add("You must define at least one era.");
				return false;
			}

			foreach (Era oEra in m_oEras)
			{
				if (!oEra.Validate(errors))
				{
					return false;
				}
			}

			return true;
		}


		private bool CheckMapFile(string mapFile)
		{
			string sDir, sFile;
			try
			{
				sDir = Path.GetDirectoryName(mapFile);
				sFile = Path.GetFileName(mapFile);
			}
			catch
			{
				sDir = "";
				sFile = "";
			}

			if (string.IsNullOrEmpty(sDir))
			{
				sDir = Path.GetDirectoryName(m_oScenario.BaseScenario.Files.Scenario);
				sFile = mapFile;
			}

			if (!File.Exists(Utility.AddTrailingSlash(sDir) + sFile))
			{
				return false;
			}

			return true;
		}
	}
}