using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace Railroads.ScenarioManager.Core
{
	public static class ScenarioVersionUpdater
	{
		public static bool UpdateLocalScenario(ApplicationSettings settings, Scenario scenario)
		{
			if (scenario.BaseScenario != null)
			{
				if (scenario.ScenarioSettings.ManagerVersion == null)
				{
					Update_Pre10(scenario, settings);
					return true;
				}
				else if (scenario.ScenarioSettings.ManagerVersion == new Version(1, 0))
				{
					Update_10(scenario);
					return true;
				}
				else if (scenario.ScenarioSettings.ManagerVersion.Value < new Version(2, 0))
				{
					//I'm not sure if any updates need to occur to get to v2.0
					//throw new NotImplementedException("Upgrades from pre 2.0 version not implemented yet.");
				}
			}

			return false;
		}


		private static void Update_Pre10(Scenario scenario, ApplicationSettings settings)
		{
			//Now that Cities, and Industries are stored in full I need to reload these files for older local scenarios

			//Remove old local files
			if (File.Exists(scenario.Files.Cities))
			{
				if (!Utility.IsSubdirectory(settings.RailroadsDirectory, scenario.Files.Cities))
				{
					File.Delete(scenario.Files.Cities);
				}
			}

			if (File.Exists(scenario.Files.Industries))
			{
				if (!Utility.IsSubdirectory(settings.RailroadsDirectory, scenario.Files.Industries))
				{
					File.Delete(scenario.Files.Industries);
				}
			}

			//Make sure base scenario is loaded
			if (scenario.BaseScenario.State != ScenarioState.Loaded)
			{
				scenario.BaseScenario.Load();
			}

			//Set file locations to base scenario files.
			//When the scenario is saved these will be recreated in local scenario folder
			ScenarioFiles oNewFiles = new ScenarioFiles(
				scenario.Files.Scenario,
				scenario.Files.Cities,
				scenario.Files.Names,
				scenario.Files.Goods,
				scenario.Files.Trains,
				scenario.BaseScenario.Files.Industries,
				scenario.BaseScenario.Files.Depots,
				scenario.BaseScenario.Files.TrainCars,
				scenario.BaseScenario.Files.Events,
				scenario.BaseScenario.Files.Eras,
				scenario.BaseScenario.Files.Decorations,
				scenario.BaseScenario.Files.Trees,
				scenario.BaseScenario.Files.Clutter,
				scenario.BaseScenario.Files.Bridges,
				scenario.BaseScenario.Files.Tunnels,
				scenario.BaseScenario.Files.Quayle);

			scenario.Files = oNewFiles;

			//Set scenario to be dirty
			scenario.IsDirty = true;
			scenario.CreateLocalFiles = true;

			scenario.ScenarioSettings.DepotsFile = Path.GetFileName(scenario.BaseScenario.Files.Depots);
			scenario.ScenarioSettings.TrainCarsFile = Path.GetFileName(scenario.BaseScenario.Files.TrainCars);
			scenario.ScenarioSettings.EventsFile = Path.GetFileName(scenario.BaseScenario.Files.Events);
			scenario.ScenarioSettings.ErasFile = Path.GetFileName(scenario.BaseScenario.Files.Eras);
			scenario.ScenarioSettings.DecorationsFile = Path.GetFileName(scenario.BaseScenario.Files.Decorations);
			scenario.ScenarioSettings.TreesFile = Path.GetFileName(scenario.BaseScenario.Files.Trees);
			scenario.ScenarioSettings.NamesFile = Path.GetFileName(scenario.BaseScenario.Files.Names);
			scenario.ScenarioSettings.ClutterFile = Path.GetFileName(scenario.BaseScenario.Files.Clutter);
			scenario.ScenarioSettings.BridgesFile = Path.GetFileName(scenario.BaseScenario.Files.Bridges);
			scenario.ScenarioSettings.TunnelsFile = Path.GetFileName(scenario.BaseScenario.Files.Tunnels);
			scenario.ScenarioSettings.QuayleFile = Path.GetFileName(scenario.BaseScenario.Files.Quayle);
		}


		private static void Update_10(Scenario scenario)
		{
			//Make sure base scenario is loaded
			if (scenario.BaseScenario.State != ScenarioState.Loaded)
			{
				scenario.BaseScenario.Load();
			}

			//Set file location to base scenario depot file.
			//When the scenario is saved this will be recreated in local scenario folder
			ScenarioFiles oNewFiles = new ScenarioFiles(
				scenario.Files.Scenario,
				scenario.Files.Cities,
				scenario.Files.Names,
				scenario.Files.Goods,
				scenario.Files.Trains,
				scenario.Files.Industries,
				scenario.BaseScenario.Files.Depots,
				scenario.BaseScenario.Files.TrainCars,
				scenario.BaseScenario.Files.Events,
				scenario.BaseScenario.Files.Eras,
				scenario.BaseScenario.Files.Decorations,
				scenario.BaseScenario.Files.Trees,
				scenario.BaseScenario.Files.Clutter,
				scenario.BaseScenario.Files.Bridges,
				scenario.BaseScenario.Files.Tunnels,
				scenario.BaseScenario.Files.Quayle);

			scenario.Files = oNewFiles;

			//Set scenario to be dirty
			scenario.IsDirty = true;

			scenario.ScenarioSettings.DepotsFile = Path.GetFileName(scenario.BaseScenario.Files.Depots);
			scenario.ScenarioSettings.TrainCarsFile = Path.GetFileName(scenario.BaseScenario.Files.TrainCars);
			scenario.ScenarioSettings.EventsFile = Path.GetFileName(scenario.BaseScenario.Files.Events);
			scenario.ScenarioSettings.ErasFile = Path.GetFileName(scenario.BaseScenario.Files.Eras);
			scenario.ScenarioSettings.DecorationsFile = Path.GetFileName(scenario.BaseScenario.Files.Decorations);
			scenario.ScenarioSettings.TreesFile = Path.GetFileName(scenario.BaseScenario.Files.Trees);
			scenario.ScenarioSettings.NamesFile = Path.GetFileName(scenario.BaseScenario.Files.Names);
			scenario.ScenarioSettings.ClutterFile = Path.GetFileName(scenario.BaseScenario.Files.Clutter);
			scenario.ScenarioSettings.BridgesFile = Path.GetFileName(scenario.BaseScenario.Files.Bridges);
			scenario.ScenarioSettings.TunnelsFile = Path.GetFileName(scenario.BaseScenario.Files.Tunnels);
			scenario.ScenarioSettings.QuayleFile = Path.GetFileName(scenario.BaseScenario.Files.Quayle);
		}
	}
}
