using System;
using System.Diagnostics;

namespace Railroads.ScenarioManager.Core
{
	public struct Version : IComparable<Version>
	{
		private int m_nMajor;
		private int m_nMinor;

		public Version(int major) : this(major, 0)
		{
		}

		public Version(int major, int minor)
		{
			m_nMajor = major;
			m_nMinor = minor;
		}



		public int Major { get { return m_nMajor; } }
		public int Minor { get { return m_nMinor; } }


		public static bool operator ==(Version left, Version right)
		{
			return left.Equals(right);
		}

		public static bool operator !=(Version left, Version right)
		{
			return !left.Equals(right);
		}

		public static bool operator <(Version left, Version right)
		{
			return (left.CompareTo(right) < 0);
		}

		public static bool operator <=(Version left, Version right)
		{
			return (left.CompareTo(right) <= 0);
		}

		public static bool operator >(Version left, Version right)
		{
			return (left.CompareTo(right) > 0);
		}

		public static bool operator >=(Version left, Version right)
		{
			return (left.CompareTo(right) >= 0);
		}

		public static Version FromString(string value)
		{
			string[] aParts = value.Split(new string[] {"."}, StringSplitOptions.RemoveEmptyEntries);
			if (aParts != null)
			{
				int nMajor = 0;
				int nMinor = 0;

				if (aParts.Length >= 2)
				{
					if (!int.TryParse(aParts[0], out nMajor))
					{
						throw new ArgumentException("The string '" + value + "' is not a valid PluginVersion");
					}
					if (!int.TryParse(aParts[1], out nMinor))
					{
						throw new ArgumentException("The string '" + value + "' is not a valid PluginVersion");
					}
				}

				if (aParts.Length == 1)
				{
					if (!int.TryParse(aParts[0], out nMajor))
					{
						throw new ArgumentException("The string '" + value + "' is not a valid PluginVersion");
					}
				}

				return new Version(nMajor, nMinor);
			}

			throw new ArgumentException("The string '" + value + "' is not a valid PluginVersion");
		}


		public override bool Equals(object right)
		{
			if (right == null)
			{
				return false;
			}

			if (this.GetType() != right.GetType())
			{
				return false;
			}

			try
			{
				Version oRhs = (Version)right;

				return ((this.m_nMajor == oRhs.m_nMajor) && (this.m_nMinor == oRhs.m_nMinor));
			}
			catch
			{
				return false;
			}
		}


		public int CompareTo(Version other)
		{
			int nResult = this.m_nMajor.CompareTo(other.m_nMajor);
			if (nResult == 0)
			{
				nResult = this.m_nMinor.CompareTo(other.m_nMinor);
			}

			return nResult;
		}


		public override int GetHashCode()
		{
			//I'm not sure if this'll will generate valid hashcodes... 
			//just using what I was as an example in MSDN Docs for ValueType.GetHashCode

			return m_nMajor.GetHashCode() ^ m_nMinor.GetHashCode();
		}


		public override string ToString()
		{
			return m_nMajor.ToString() + "." + m_nMinor.ToString();
		}
	}
}